/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Glass_Glass.h"
#include "glassjava.h"

static Glass *getGlass(JNIEnv *jenv, jobject jobj) {
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
    	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return JNI_FALSE;
	}
	return (Glass *)jenv->GetLongField(jobj, fid);
}


/*
 * Class:     Glass_Glass
 * Method:    getNodeId
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_Glass_Glass_getNodeId
  (JNIEnv *jenv, jobject jobj) {
	Glass *g = getGlass(jenv, jobj);
	return (int)g->getNodeId();
}

/*
 * Class:     Glass_Glass
 * Method:    getNodeRelation
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_Glass_Glass_getNodeRelation
  (JNIEnv *jenv, jobject jobj, jint node) {
	Glass *g = getGlass(jenv, jobj);
	return (int)g->getNodeRelation(node);
}

/*
 * Class:     Glass_Glass
 * Method:    setCurrent
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_Glass_setCurrent
  (JNIEnv *jenv, jobject jobj) {
	Glass *g = getGlass(jenv, jobj);
	return g->setCurrent();
}

/*
 * Class:     Glass_Glass
 * Method:    lockCurrent
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_Glass_lockCurrent
  (JNIEnv *jenv, jobject jobj) {
	Glass *g = getGlass(jenv, jobj);
	return g->lockCurrent();
}

/*
 * Class:     Glass_Glass
 * Method:    unlockCurrent
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_Glass_unlockCurrent
  (JNIEnv *jenv, jobject jobj) {
	Glass *g = getGlass(jenv, jobj);
	return g->unlockCurrent();
}

/*
 * Class:     Glass_Glass
 * Method:    getTotalNodes
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_Glass_Glass_getTotalNodes
  (JNIEnv *jenv, jobject jobj) {
	Glass *g = getGlass(jenv, jobj);
	return g->getTotalNodes();
}
