/**
 *  Java binding for libGlass
 *  Copyright (C) 2003,2004
 *  Bruno Barberi Gnecco and Marcelo de Paiva Guimaraes
 *  Copyright (C) 2005-2010 Corollarium Tecnologia
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Glass_GlassClient.h"
#include "glassjava.h"

/*
 * Class:     Glass_GlassClient
 * Method:    createGlassClient
 * Signature: (LGlass/Protocol;Ljava/lang/String;)I
 */
JNIEXPORT jlong JNICALL Java_Glass_GlassClient_createGlassClient
(JNIEnv *jenv, jobject jobj, jobject protocol, jstring servername, jstring nickname, jboolean tryuntilconnect) {
	GlassClient		*c;
	Protocol		*p;
	const char		*str;
	const char		*nick;
	// como transcrever jboolean tryuntilconnect pra JNI ?

	// convert protocol
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(protocol);
	fid = jenv->GetFieldID(cls, "pointer", "J");
	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return 0;
	}
	p = (Protocol *)jenv->GetLongField(protocol, fid);

	str = jenv->GetStringUTFChars(servername, 0);
	nick = jenv->GetStringUTFChars(nickname, 0);

	try {
		// tryuntilconnect deve ser mapeado aqui
		c = new GlassClient(p, str, nick, false);
	}
	catch (Exception e) {
		jenv->ReleaseStringUTFChars(servername, str);
		jenv->ReleaseStringUTFChars(nickname, nick);

		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

	return (jlong)c;
}

static GlassClient *getGlassClient(JNIEnv *jenv, jobject jobj) {
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return JNI_FALSE;
	}
	return (GlassClient *)jenv->GetLongField(jobj, fid);
}

/*
 * Class:     Glass_GlassClient
 * Method:    destroyGlassClient
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_GlassClient_destroyGlassClient
(JNIEnv *jenv, jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	delete c;
}

/*
 * Class:     Glass_GlassClient
 * Method:    getTotalNodes
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_Glass_GlassClient_getTotalNodes
(JNIEnv *jenv, jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	return c->getTotalNodes();
}

/*
 * Class:     Glass_GlassClient
 * Method:    getClientList
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL Java_Glass_GlassClient_getClientList
(JNIEnv *jenv, jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	std::map<nodeId, nodeNickName> nodes(c->getClientList());
	int i = 0, size = nodes.size();

	jintArray jnodes= jenv->NewIntArray(size);
	jint *array = new jint[size];
	for (std::map<nodeId, nodeNickName>::const_iterator it(nodes.begin()); it != nodes.end(); it++, i++) {
		array[i] = (jint)(*it).first;
	}
	jenv->SetIntArrayRegion(jnodes, 0, size, array);
	delete[] array;

	return jnodes;
}

/*
 * Class:     Glass_GlassClient
 * Method:    _getClientList
 * Signature: ()[Ljava/lang/String;
 */
JNIEXPORT jobjectArray JNICALL Java_Glass_GlassClient__1getClientList
(JNIEnv *jenv , jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	std::map<nodeId, nodeNickName> nodes(c->getClientList());

	jobjectArray ret;  

	int i = 0;

	ret= (jobjectArray)jenv->NewObjectArray(5,  
			jenv->FindClass("java/lang/String"),  
			jenv->NewStringUTF(""));  

	for(std::map<nodeId, nodeNickName>::const_iterator it(nodes.begin()); it != nodes.end(); it++, i++) {  
		const char *tst1;
		const char *tst2;

		std::string msg1;
		std::stringstream out;
		out << (*it).first;
		msg1 = out.str();

		std::string msg2;
		out << (*it).second;;
		msg2 = out.str();

		tst1=msg1.c_str();
		tst2=msg2.c_str();

		jenv->SetObjectArrayElement(  
				ret,i,jenv->NewStringUTF(tst1));  
		i++;
		jenv->SetObjectArrayElement(  
				ret,i,jenv->NewStringUTF(tst2)); 

	}

	return ret;

}

/*
 * Class:     Glass_GlassClient
 * Method:    isMaster
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_Glass_GlassClient_isMaster
(JNIEnv *jenv, jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	return c->isMaster();
}

/*
 * Class:     Glass_GlassClient
 * Method:    killAll
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_GlassClient_killAll
(JNIEnv *jenv, jobject jobj) {
	GlassClient *c = getGlassClient(jenv, jobj);
	return c->killAll();
}
