/**
 *  Java binding for libGlass
 *  Copyright (C) 2003-2010
 *  Corollarium Technologies
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  @author      Corollarium Technologies
 *  @version     %I%, %G%
 *  @since       1.0
 */

package Glass;

import java.util.HashMap;

/**
 * 
 * @author corollarium
 *
 */
public class GlassClient extends Glass {
	protected long pointer;
	private Protocol protocol;

	/**
	 * Wrapper for C++ constructor.
	 * 
	 * @param p The protocol class
	 * @param server The server (ip or hostname
	 * @return The pointer to the C++ object, cast to long.
	 */
	private native long createGlassClient(Protocol p, 
			String server, String nickname, boolean tryuntilconnect); 

	/**
	 * Wrapper for C++ destructor.
	 */
	private native void destroyGlassClient();

	/**
	 * Glass client constructor.
	 *  
	 * @param p The protocol to use.
	 * @param server The server name or IP.
	 * @throws GlassException
	 */
	public GlassClient(Protocol p, String server, String nickname, boolean tryuntilconnect) throws GlassException {
		super();
		try {
			this.pointer = createGlassClient(p, server, nickname, tryuntilconnect);
			this.protocol = p;
		}
		catch (Exception e) {
			throw new GlassException(e);
		}
	}

	protected void finalize() throws Throwable {
		try {
			destroyGlassClient();
		}
		finally {
			super.finalize();
		}
	}

	public Protocol getProtocol() {
		return protocol;
	}

	/**
	 * Returns the number \e CLIENT nodes connected to the network.
	 * 
	 * This function returns the number \e CLIENT nodes connected to the 
	 * network. There's only one server around for each client.
	 * 
	 * The returned number is not guaranteed to be valid, as nodes may
	 * join or leave at any time. 
	 * 
	 * @return The number of CLIENT nodes connected to the network, or 
	 * zero if an error occurred.
	 */
	public native int getTotalNodes();

	public native String[] _getClientList();

	/**
	 * Returns the list of all client nodes.
	 * 
	 * The returned value is not guaranteed to be valid, as nodes may
	 * join or leave at any time. 
	 *
	 * @return A set with all valid nodeIds at the moment the call 
	 * was executed, or an empty list if an error occurred.
	 */
	public HashMap<nodeId, nodeNickName> getClientList(){
		HashMap<nodeId, nodeNickName> map = new HashMap<nodeId, nodeNickName>();
		String[] array = this._getClientList();
		nodeId node;
		nodeNickName nick;
		for(int i = 0; i< array.length; i++){
			node = new nodeId(Integer.parseInt(array[i]));
			nick = new nodeNickName(array[i+1]);
			map.put(node, nick);
			i=i+2;
		}
		return map;
	}


	/**
	 * Is this node the master?
	 * 
	 * @return true if this is the node with lowest id. false otherwise.
	 */
	public native boolean isMaster();

	/**
	 * Close all nodes, including this one.
	 */
	public native void killAll();
}
