 /**
 *  Java binding for libGlass
 *  Copyright (C) 2003,2004
 *  Bruno Barberi Gnecco and Marcelo de Paiva Guimaraes
 *  Copyright (C) 2005-2011 Corollarium Tecnologia
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Glass_GlassServer.h"
#include "glassjava.h"

/*
 * Class:     Glass_GlassServer
 * Method:    createGlassServer
 * Signature: (LGlass/Protocol;)I
 */
JNIEXPORT jlong JNICALL Java_Glass_GlassServer_createGlassServer
  (JNIEnv *jenv, jobject jobj, jobject protocol) {
	GlassServer *s;
	Protocol *p;

	// TODO: convert protocol
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(protocol);

	fid = jenv->GetFieldID(cls, "pointer", "J");
    if (fid == NULL) {
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, "Missing field pointer");
		}
		return 0;
	}
	p = (Protocol *)jenv->GetLongField(protocol, fid);

	try {
		s = new GlassServer(p);
	}
	catch (Exception e) {
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

	return (jlong)s;

}

/*
 * Class:     Glass_GlassServer
 * Method:    destroyGlassServer
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_GlassServer_destroyGlassServer
  (JNIEnv *jenv, jobject jobj) {
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
    	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return;
	}
	GlassServer *s = (GlassServer *)jenv->GetLongField(jobj, fid);
	delete s;
}
