 /**
 *  Java binding for libGlass
 *  Copyright (C) 2003,2004
 *  Bruno Barberi Gnecco and Marcelo de Paiva Guimaraes
 *  Copyright (C) 2005-2010 Corollarium Tecnologia
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Glass_GlassServerPeer.h"
#include "glassjava.h"

/*
 * Class:     Glass_GlassServerPeer
 * Method:    createGlassServerPeer
 * Signature: (LGlass/Protocol;Ljava/lang/String;I)J
 */
JNIEXPORT jlong JNICALL Java_Glass_GlassServerPeer_createGlassServerPeer
  (JNIEnv *jenv, jobject jobj, jobject protocol, jstring peerj, jint peerport) {
	GlassServerPeer *s;
	Protocol *p;

    const char *peerhost_c = jenv->GetStringUTFChars(peerj, NULL);
    if (peerhost_c == NULL) {
        return NULL; /* OutOfMemoryError already thrown */
    }

    // TODO: convert protocol
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(protocol);

	fid = jenv->GetFieldID(cls, "pointer", "J");
	if (fid == NULL) {
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ReleaseStringUTFChars(peerj, peerhost_c);
			jenv->ThrowNew(except, "Missing field pointer");
		}
		return 0;
	}
	p = (Protocol *)jenv->GetLongField(protocol, fid);

	try {
		s = new GlassServerPeer(p, peerhost_c, peerport);
	}
	catch (Exception e) {
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ReleaseStringUTFChars(peerj, peerhost_c);
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

    jenv->ReleaseStringUTFChars(peerj, peerhost_c);

	return (jlong)s;
}

/*
 * Class:     Glass_GlassServerPeer
 * Method:    destroyGlassServerPeer
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_GlassServerPeer_destroyGlassServerPeer
   (JNIEnv *jenv, jobject jobj) {
	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
  	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return;
	}
	GlassServerPeer *s = (GlassServerPeer *)jenv->GetLongField(jobj, fid);
	delete s;
}
