 /**
  *  Java binding for libGlass
  *  Copyright (C) 2003-2010
  *  Corollarium Technologies
  * 
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  @author      Corollarium Technologies
  *  @version     %I%, %G%
  *  @since       1.0
  */

package Glass;

/**
 * Glass server class
 * 
 * @author corollarium
 *
 */
public class GlassServerPeer extends Glass {
	private Protocol protocol;
	
	/**
	 * Wrapper for C++ constructor.
	 * 
	 * @param p The protocol class
	 * @return The pointer to the C++ object, cast to long.
	 */
	private native long createGlassServerPeer(Protocol p, String peer, int peerport);

	/**
	 * Wrapper for C++ destructor.
	 */
	private native void destroyGlassServerPeer();
	
	/**
	 * Glass server constructor. Never returns if successful.
	 *   
	 * @param p The protocol.
	 * @throws GlassException
	 */
	public GlassServerPeer(Protocol p, String peer, int peerport) throws GlassException {
		try {
			this.pointer = createGlassServerPeer(p, peer, peerport);
			this.protocol = p;
		}
		catch (Exception e) {
			throw new GlassException(e);
		}
	}
	
	/**
	 * @see #GlassServer(Protocol, String, int)
	 */
	public GlassServerPeer(Protocol p, String peer) throws GlassException {
		this(p, peer, p.getListenPort());
	}

	protected void finalize() throws Throwable {
		try {
			destroyGlassServerPeer();
		}
		finally {
			super.finalize();
		}
	}
}