 /**
 *  Java binding for libGlass
 *  Copyright (C) 2003,2004
 *  Bruno Barberi Gnecco and Marcelo de Paiva Guimaraes
 *  Copyright (C) 2005-2011 Corollarium Tecnologia
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Glass_Remote.h"
#include "glassjava.h"


static Remote<char *, rawPack, rawUnpack>* getRemotePointer(JNIEnv *jenv, jobject jobj) {
	// TODO: cache ids

	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
    	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return JNI_FALSE;
	}
	return (Remote<char *, rawPack, rawUnpack> *)jenv->GetLongField(jobj, fid);
}


/*
 * Class:     Glass_Remote
 * Method:    createRemote
 * Signature: (Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_Glass_Remote_createRemote
  (JNIEnv *jenv, jobject jobj, jstring name) {
	Remote<char *, rawPack, rawUnpack> *s;
	const char *str;

	// convert the Java String to use it in C
	str = jenv->GetStringUTFChars(name, 0);

	try {
		s = new Remote<char *, rawPack, rawUnpack>(str, NULL, 0);
	}
	catch (Exception e) {
		jenv->ReleaseStringUTFChars(name, str);
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

	jenv->ReleaseStringUTFChars(name, str);

	return (jlong)s;
}

/*
 * Class:     Glass_Remote
 * Method:    destroyRemote
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_Remote_destroyRemote
  (JNIEnv *jenv, jobject jobj) {
	Remote<char *, rawPack, rawUnpack> *s = getRemotePointer(jenv, jobj);
	delete s;
}

/*
 * Class:     Glass_Remote
 * Method:    _getData
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_Glass_Remote__1getData
  (JNIEnv *jenv, jobject jobj) {
	Remote<char *, rawPack, rawUnpack> *s = getRemotePointer(jenv, jobj);

	// TODO: len
	unsigned int len = 0;
	const char *data = s->getData();

	//std::cerr << "=====len: " << len << "0x" << std::hex << (int)data[0] << std::endl;
	jbyteArray jvalue = jenv->NewByteArray(len);
	jenv->SetByteArrayRegion(jvalue, 0, len, (jbyte *)data);
	return jvalue;
}

/*
 * Class:     Glass_Remote
 * Method:    _getValue
 * Signature: (I)[B
 */
JNIEXPORT jbyteArray JNICALL Java_Glass_Remote__1getValue
(JNIEnv *jenv, jobject jobj, jint id) {
	Remote<char *, rawPack, rawUnpack> *s = getRemotePointer(jenv, jobj);

	// TODO: test
	unsigned int len = 0;
	char *data;
	try {
		data = s->getValue(id, len);
	}
	catch (Exception e) {
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

	//std::cerr << "=====len: " << len << "0x" << std::hex << (int)data[0] << std::endl;
	jbyteArray jvalue = jenv->NewByteArray(len);
	jenv->SetByteArrayRegion(jvalue, 0, len, (jbyte *)data);
	return jvalue;
}

/*
 * Class:     Glass_Remote
 * Method:    _setValue
 * Signature: ([BI)Z
 */
JNIEXPORT jboolean JNICALL Java_Glass_Remote__1setValue
(JNIEnv *jenv, jobject jobj, jbyteArray data, jint id) {
	Remote<char *, rawPack, rawUnpack> *s = getRemotePointer(jenv, jobj);

	jsize len = jenv->GetArrayLength(data);
	jbyte *jBytes = jenv->GetByteArrayElements(data, 0);
	char *cdata = (char *)jBytes;
	bool ret = s->setValue(id, cdata, (unsigned int)len, false);

	jenv->ReleaseByteArrayElements(data, jBytes, JNI_ABORT);

	return ret;
}
