 /**
 *  Java binding for libGlass
 *  Copyright (C) 2003,2004
 *  Bruno Barberi Gnecco and Marcelo de Paiva Guimaraes
 *  Copyright (C) 2005-2010 Corollarium Tecnologia
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Glass_Shared.h"
#include "glassjava.h"

Shared<char *, rawPack, rawUnpack>* getSharedPointer(JNIEnv *jenv, jobject jobj) {
	// TODO: cache ids

	jfieldID fid;
	jclass cls = jenv->GetObjectClass(jobj);

	fid = jenv->GetFieldID(cls, "pointer", "J");
    	if (fid == NULL) {
		DEBUG("ERROR" << __LINE__);
		return JNI_FALSE;
	}
	return (Shared<char *, rawPack, rawUnpack> *)jenv->GetLongField(jobj, fid);
}


/*
 * Class:     Glass_Shared
 * Method:    createShared
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jlong JNICALL Java_Glass_Shared_createShared
  (JNIEnv *jenv, jobject jobj, jstring name, jint propagation) {
	Shared<char *, rawPack, rawUnpack> *s;
	const char *str;

	// convert the Java String to use it in C
	str = jenv->GetStringUTFChars(name, 0);

	try {
		propagationMode mode = static_cast<propagationMode>(propagation);
		if (mode != Local && mode != Global) {
			throw Exception("Invalid propagation mode");
		}
		s = new Shared<char *, rawPack, rawUnpack>(str, NULL, 0, mode);
	}
	catch (Exception e) {
		jenv->ReleaseStringUTFChars(name, str);  
		jclass except = jenv->FindClass("java/lang/Exception");
		if (except != 0) {
			jenv->ThrowNew(except, e.getMessage().c_str());
		}
		return 0;
	}

	jenv->ReleaseStringUTFChars(name, str);

	return (jlong)s;
}

/*
 * Class:     Glass_Shared
 * Method:    destroyShared
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Glass_Shared_destroyShared
  (JNIEnv *jenv, jobject jobj) {
	Shared<char *, rawPack, rawUnpack> *s = getSharedPointer(jenv, jobj);
	delete s;
}

/*
 * Class:     Glass_Shared
 * Method:    hasChanged
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_Glass_Shared_hasChanged
  (JNIEnv *jenv, jobject jobj) {
	Shared<char *, rawPack, rawUnpack> *s = getSharedPointer(jenv, jobj);
	return s->hasChanged();
}
/*
 * Class:     Glass_Shared
 * Method:    sendUpdate
 * Signature: ([B)Z
 */
JNIEXPORT jboolean JNICALL Java_Glass_Shared_sendUpdate
  (JNIEnv *jenv, jobject jobj, jbyteArray data) {
	Shared<char *, rawPack, rawUnpack> *s = getSharedPointer(jenv, jobj);

	jsize len = jenv->GetArrayLength(data);
	jbyte *jBytes = jenv->GetByteArrayElements(data, 0);
	char *cdata = (char *)jBytes;
	s->assign(cdata, (unsigned int)len, false, false);
	bool ret = s->sendUpdate();

	jenv->ReleaseByteArrayElements(data, jBytes, JNI_ABORT);

	return ret;
}

/*
 * Class:     Glass_Shared
 * Method:    _getUpdate
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_Glass_Shared__1getUpdate
  (JNIEnv *jenv, jobject jobj) {
	Shared<char *, rawPack, rawUnpack> *s = getSharedPointer(jenv, jobj);
	return s->getUpdate();
}

/*
 * Class:     Glass_Shared
 * Method:    _getData
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_Glass_Shared__1getData
(JNIEnv *jenv, jobject jobj) {
	Shared<char *, rawPack, rawUnpack> *s = getSharedPointer(jenv, jobj);

	unsigned int len = 0;
	const char *data;
	data = s->getData();
	len = s->getLen();

	//std::cerr << "=====len: " << len << "0x" << std::hex << (int)data[0] << std::endl;
	jbyteArray jvalue = jenv->NewByteArray(len);
	jenv->SetByteArrayRegion(jvalue, 0, len, (jbyte *)data);
	return jvalue;
}
