 /**
  *  Java binding for libGlass
  *  Copyright (C) 2003-2010
  *  Corollarium Technologies
  * 
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  @author      Corollarium Technologies
  *  @version     %I%, %G%
  *  @since       1.0
  */

package Glass;

/**
 * TCP protocol class.
 * 
 * @author corollarium
 *
 */
public class TCP extends Protocol {
	static { // load the JNI library
		System.loadLibrary("glassjava");
	}

	static final int defaultPort = 9191;
	
	private int listenPort = defaultPort;
	
	private native long createTCP(int port);

	/**
	 * Destructor 
	 */
	private native void destroyTCP();

	/**
	 * TCP constructor
	 * 
	 * @param port The port to listen to.
	 * @throws GlassException 
	 */
	public TCP(int port) throws GlassException {
		this.listenPort = port;

		try {
			this.pointer = this.createTCP(port);
		}
		catch (java.lang.UnsatisfiedLinkError e) {
			throw new GlassException("Glass library not found");
		}
		catch (Exception e) {
			throw new GlassException(e);
		}
	}

	/**
	 * @see #TCP(int)
	 * @throws GlassException
	 */
	public TCP() throws GlassException {
		this(TCP.defaultPort);
	}

	protected void finalize() throws Throwable {
		try {
			destroyTCP();
		}
		finally {
			super.finalize();
		}
	}

	@Override
	int getListenPort() {
		return this.listenPort;
	}

}