 /**
  *  Java binding for libGlass
  *  Copyright (C) 2003-2010
  *  Corollarium Technologies
  * 
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  @author      Corollarium Technologies
  *  @version     %I%, %G%
  *  @since       1.0
  */

package Glass;

/**
 * TCP protocol class.
 * 
 * @author corollarium
 *
 */
public class UnixSocket extends Protocol {
	static { // load the JNI library
		System.loadLibrary("glassjava");
	}

	private native long createUnixSocket();

	/**
	 * Destructor 
	 */
	private native void destroyUnixSocket();

	/**
	 * TCP constructor
	 * 
	 * @param port The port to listen to.
	 * @throws GlassException 
	 */
	public UnixSocket() throws GlassException {
		try {
			this.pointer = this.createUnixSocket();
		}
		catch (java.lang.UnsatisfiedLinkError e) {
			throw new GlassException("Glass library not found");
		}
		catch (Exception e) {
			throw new GlassException(e);
		}
	}

	protected void finalize() throws Throwable {
		try {
			destroyUnixSocket();
		}
		finally {
			super.finalize();
		}
	}
	
	@Override
	int getListenPort() {
		return 0;
	}
}