/**
 *  Java binding for libGlass
 *  Copyright (C) 2003-2010
 *  Corollarium Technologies
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  @author      Corollarium Technologies
 *  @version     %I%, %G%
 *  @since       1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import javax.media.opengl.awt.GLCanvas;
import com.sun.opengl.util.Animator;
import static javax.media.opengl.GL.*;
import java.awt.Frame;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import Glass.*;

public class SimpleSphere implements GLEventListener, KeyListener {
	Frame fr;
	static GLCanvas canvas = new GLCanvas();
	static GLU glu = new GLU();
	/**
	 * Glass Variables
	 */
	protected GlassClient g;
	protected Barrier swaplock, datalock;
	protected Shared<Integer> deg;
	protected Alias<String> view_set;
	private int target_node = 3;
	private int tmprot = 0;

	public SimpleSphere(String server) {
		glassInitialization(server);
		fr = new Frame("Around - Java - libglass");
		canvas.addGLEventListener(this);
		fr.add(canvas);
		Animator anim = new Animator(canvas);
		fr.setSize(640, 480);
		fr.setVisible(true);
		anim.start();
		fr.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				super.windowClosing(e);
				System.exit(0);
			}
		});
	}

	@Override
	public void init(GLAutoDrawable arg0) {
		GL2 gl = arg0.getGL().getGL2();
		System.err.println("INIT GL IS: " + gl.getClass().getName());
		gl.glEnable(gl.GL_DEPTH_TEST);
		gl.glEnable(gl.GL_CULL_FACE);
		gl.glEnable(gl.GL_LIGHTING);
		gl.glEnable(gl.GL_LIGHT0);
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	}

	@Override
	public void display(GLAutoDrawable arg0) {
		final GL2 gl = arg0.getGL().getGL2();
		final float light_pos[] = { (float) 10.0, (float) 16.0, (float) 0.0,
				(float) 0.0 };
		final float colors[][] = {
				{ (float) 1.0, (float) 0.0, (float) 0.0, (float) 1.0 },
				{ (float) 0.0, (float) 1.0, (float) 0.0, (float) 1.0 },
				{ (float) 0.0, (float) 0.0, (float) 1.0, (float) 1.0 } };
		float white[] = { (float) 1., (float) 1., (float) 1., (float) 1. };
		int i = 0, j;

		// send updates, synchronize and update variables.
		try {
			if (g.isMaster()) {
				deg.setData(deg.getData() + 1);
				if (deg.getData() > 360) {
					deg.setData(0);
				}
				deg.sendUpdate();
			}

			datalock.sync();
			deg.getUpdate();
		} catch (ClassNotFoundException e) {

		} catch (IOException e) {
			// TODO
		}
		gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();

		/*
		 * Set the view
		 */
		String view = view_set.getData();
		if (view == "front") {
		} else if (view == "left") {
			gl.glRotated(90, 0, -1, 0);
		} else if (view == "right") {
			gl.glRotated(90, 0, 1, 0);
		} else if (view == "back") {
			gl.glRotated(180, 0, -1, 0);
		}

		GLUquadric quadric = glu.gluNewQuadric();
		glu.gluQuadricTexture(quadric, true);
		gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_POSITION, light_pos, 0);
		for (i = 60; i >= -60; i -= 20) {
			for (j = 0; j < 3; j++) {
				gl.glPushMatrix();
				// apply the rotation
				gl.glRotated((deg.getData() + j * 120) % 360, 0, 1, 0);
				gl.glTranslatef(0, i, -50);
				gl.glMaterialfv(gl.GL_FRONT, GL2.GL_SPECULAR, white, 0);
				gl.glMaterialf(gl.GL_FRONT, GL2.GL_SHININESS, (float) 20.0);
				gl.glMaterialfv(gl.GL_FRONT, GL2.GL_AMBIENT_AND_DIFFUSE,
						colors[j], 0);

				glu.gluSphere(quadric, 8.0, 10, 10);

				gl.glPopMatrix();
			}
			swaplock.sync();
		}
		glu.gluDeleteQuadric(quadric);
	}

	@Override
	public void dispose(GLAutoDrawable drawable) {

	}

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int width,
			int height) {
		GL2 gl = drawable.getGL().getGL2();
		if (height <= 0) {
			height = 1;
		}
		float h = (float) width / (float) height;
		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(50.0f, h, 1.0, 1000.0);
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
			System.exit(0);
		}
		// TODO
		int key = e.getKeyCode();
		switch (key) {
		case '3':
		case '4':
		case '5':
		case '6':
			target_node = key - '3' + 3;
			break;
		case 'f':
		case 'F':
			view_set.associate(target_node, "front");
			break;
		case 'r':
		case 'R':
			view_set.associate(target_node, "right");
			break;
		case 'l':
		case 'L':
			view_set.associate(target_node, "left");
			break;
		case 'b':
		case 'B':
			view_set.associate(target_node, "back");
			break;
		default:
			break;
		}

	}

	public void keyReleased(KeyEvent e) {
	}

	public void keyTyped(KeyEvent e) {
	}

	private void glassInitialization(String server) {
		try {
			if (server == "") {
				System.out.println("Starting server");
				GlassServer gs = new GlassServer(new TCP());
			} else {
				System.out.println("Connecting to: " + server);
				g = new GlassClient(new TCP(), server);
			}

			/* create a barrier for swaplock */
			this.datalock = new Barrier(1);
			this.swaplock = new Barrier(2);

			/* create our shared data, and set its value */
			this.deg = new Shared<Integer>("deg");
			deg.setData(0);
			if (g.getNodeId() == 3) { // first node, set value
				try {
					this.deg.sendUpdate();
				} catch (IOException e) {
					// TODO
				}
			}

			/* create and set our view set function alias */
			view_set = new Alias<String>("view_set", "front");
			view_set.addAlias("front", "front");
			view_set.addAlias("right", "right");
			view_set.addAlias("left", "left");
			view_set.addAlias("back", "back");

			if (g.getNodeId() == 3) {
				view_set.associate(3, "front");
				view_set.associate(4, "right");
				view_set.associate(5, "left");
				view_set.associate(6, "back");
			}

		} catch (GlassException e) {
			System.out.println("Error: glass initialization: " + e);
			System.exit(1);
		}
	}

	public static void main(String[] args) {
		String server = "";
		if (args.length != 0) {
			server = args[0];
		}
		canvas.addKeyListener(new SimpleSphere(server));
	}

}
