/**
 *  Java binding for libGlass
 *  Copyright (C) 2003-2010
 *  Corollarium Technologies
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  @author      Corollarium Technologies
 *  @version     %I%, %G%
 *  @since       1.0
 */

import Glass.*;
import java.io.IOException;

class TestEvent {
	public static final int EVENTS_TO_SEND = 50;

	public static void main(String[] args) {
		/*
		 * Setup client
		 */
		String server;
		if (args.length == 0) {
			server = "localhost";
		} else {
			server = args[0];
		}
		System.out.println("Connecting to: " + server);

		GlassClient g = null;
		try {
			g = new GlassClient(new TCP(), server, "", false);
		} catch (GlassException e) {
			System.out.println("Error: Client initialization: " + e);
			return;
		}

		/*
		 * Run
		 */
		try {
			int eventnum = 0;
			Event<Integer> eventint = new Event<Integer>("integer");
			Event<String> eventstr = new Event<String>("string");

			if (g.isMaster()) {
				System.out.println("I'm master");
				for (int i = 0; i < EVENTS_TO_SEND; i++) {
					if (eventint.enqueueEvent(eventnum++) == false) {
						System.out.println("Error enqueuing int");
					}else{
						System.out.println("Enqueuing int : "+eventnum);
					}
					if (eventstr.enqueueEvent("Glass library event test "+eventnum) == false) {
						System.out.println("Error enqueuing string");
					}else{
						System.out.println("Eenqueuing str :"+"Glass library event test "+eventnum);
					}
					Thread.sleep(1000);
				}
				g.killAll();
			} else
				while (true) {
					if (eventint.isEmpty()) {
						System.out.println("Int is empty");
						Thread.sleep(1000);
					} else {
						try {
							System.out.println("Received event "
								+ eventint.getEvent());
						}
						catch (EmptyException e) {
							// no event in queue. ignore.
						}
					}
					if (eventstr.isEmpty()) {
						System.out.println("string is empty");
						Thread.sleep(1000);
					} else {
						try {
							System.out.println("Received event "
								+ eventstr.getEvent());
						}
						catch (EmptyException e) {
							// no event in queue. ignore.
						}
					}
				}
		} 
		catch (GlassException e) {
			System.out.println("Error: class initialization: " + e);
			return;
		} 
		catch (InterruptedException e) {
		} 
		catch (IOException e) {
		}
		catch (ClassNotFoundException e) {
		}

		return;
	}
}
