 /**
  *  Java binding for libGlass
  *  Copyright (C) 2003-2010
  *  Corollarium Technologies
  * 
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  @author      Corollarium Technologies
  *  @version     %I%, %G%
  *  @since       1.0
  */

import java.io.IOException;
import java.io.Serializable;

import Glass.*;

class TestEventObject {
	public static final int EVENTS_TO_SEND = 15;

	public static class DataExample implements Serializable {
		private static final long serialVersionUID = 1L;
		
		private int i;
		private float f;

		public DataExample(int i, float f) {
			this.i = i;
			this.f = f;
		}

		public int getI() {
			return i;
		}

		public void setI(int i) {
			this.i = i;
		}

		public float getF() {
			return f;
		}

		public void setF(float f) {
			this.f = f;
		}
	
		public boolean equals(DataExample e) {
			return (this.f == e.f && this.i == e.i);
		}

		public String toString() {
			return "i=" + i + ", f=" + f;
		}
	}

	public static void main(String[] args) {
		/*
		 * Setup client 
		 */
		String server;
		if (args.length == 0) {
			server = "localhost";
		}
		else {
			server = args[0];
		}
		System.out.println("Connecting to: " + server);
		
		GlassClient g = null;
		try {
			g = new GlassClient(new TCP(), server, "", false);
		} catch(GlassException e){
			System.out.println("Error: Client initialization: "+ e);
		}
		System.out.println("Connected: " + server);

		/*
		 * Run
		 */
		try {
			int	eventnum = 0;
			Event<DataExample> event = new Event<DataExample>("data");
			DataExample data = new DataExample(10, 2.0f);
			DataExample newdata;
	
			if (g.isMaster()) {
				System.out.println("I'm master");
				for (int i = 0; i < EVENTS_TO_SEND; i++) {
					if (event.enqueueEvent(data) == false) {
						System.out.println("Error enqueuing int");
					}
					System.out.println("Enqueued");
					Thread.sleep(1000);
				}
				g.killAll();
			}
			else while(true) {
				System.out.println("I'm client");
				try{
					newdata = (DataExample)event.getEvent();
					if (data.equals(newdata)) {
						System.out.println("Events match: " + data + " and " + newdata);
					}
					else {
						System.out.println("Events DO NOT match: " + data + " and " + newdata);
					}
				}
				catch (EmptyException e) {
					System.out.println("Int is empty");
					Thread.sleep(1000);
				} 
			}
		}
		catch (GlassException e) {
			System.out.println("Error: class initialization: " + e);
			return;
		} 
		catch (InterruptedException e) {
		} 
		catch (IOException e) {
		}
		catch (ClassNotFoundException e) {
		}
		
		return;
	}
}
