/**
 *  Java binding for libGlass
 *  Copyright (C) 2003-2010
 *  Corollarium Technologies
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  @author      Corollarium Technologies
 *  @version     %I%, %G%
 *  @since       1.0
 */

import Glass.*;
import java.io.IOException;

class TestShared {
	public static final int EVENTS_TO_SEND = 5;

	public static void main(String[] args) {
		/*
		 * Setup client
		 */
		Shared<Integer> sharedint;
		GlassServer s = null;
		GlassClient g = null;
		try {
			if (args.length == 0) {
				s = new GlassServer(new TCP());
			} else {
				System.out.println("Connecting to: " + args[0]);
				g = new GlassClient(new TCP(), args[0], "", false);
				System.out.println("Client started ");
			}
		} catch (GlassException e) {
			System.out.println("Error: Client initialization: " + e);
		}

		// am I master?
		if (g.isMaster()) {
			System.out.println("I am Master");
		} else {
			System.out.println("I am not Master");
		}
		System.out.println("My node number is : " + g.getNodeId());

		// Create the data
		try {
			System.out.println("Creating shared data ");
			// sharedint = new Shared<Integer>("sharedint");
			sharedint = new Shared<Integer>("sharedint", 0);

		} catch (GlassException e) {
			System.out.println("Error: creating shared data: " + e);
			return;
		}

		System.out.println("Client started ");
		int i = 0;
		System.out.println("Updating ...");
		while (true) {
			if (g.isMaster()) {
				System.out.println("Updating sharedint...");
				// update the data
				i++;
				try {
					// sets the new data
					sharedint.setData(i);
					// propagate to other clients
					sharedint.sendUpdate();
				} catch (java.io.IOException e) {
					System.out.println("Error: Shared data propagation: " + e);
					return;
				}
			} else{
				// gets the current value. Explicit synchronization
				System.out.println("getting update");
				try {
					sharedint.getUpdate();
					// uses the value. This call is instantaneous, data is cached.
					System.out.println("new value: " + sharedint.getData());
				} catch (ClassNotFoundException e) {
					System.out.println("Error: getting update: " + e);
					// return;
				}
			}

			try {
				Thread.sleep(1000); // milliseconds
			} catch (InterruptedException e) {
				System.out.println("Error: sleep  " + e);
				return;
			}
		}
		// return;
	}
}
