/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies

                                                                                                   
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
                                                                                                   
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
                                                                                                   
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/******************************************************************************
Multiprojection example
                                                                                
Program description:
     'aroundcontrol' is a multiprojection example which is controled by 
     a java application
                                                                                
Dependencies:
                                                                                
Authors:
	Bruno Barberi Gnecco
                                                                                
History:
	22 June 2003:Initial implementation: (Bruno Barberi Gnecco)
    22 June 2003: added event treatment (Marcelo de Paiva Guimaraes)

                                                                                
Return codes:
0       Successful
                                                                                
***************************************************************************/


#include "libglass.h"
#include <iostream>
#include <stdio.h>
#include <GL/glut.h>

using namespace libglass;

Glass		*g;
Barrier 	*datalock;
Barrier 	*swaplock;
Shared<int, rawPack, rawUnpack> *deg;
Shared<int, rawPack, rawUnpack> *pos;
Event<int, rawPack, rawUnpack> *event;


Alias<void (*)(void)> *view_set;
nodeId		node_set = 3;

int winW = 320, winH = 240;

void front (void) {
	//std::cerr << __PRETTY_FUNCTION__ << std::endl;
}
void left (void) {
	//std::cerr << __PRETTY_FUNCTION__ << std::endl;
	glRotated(90, 0, -1, 0);
}
void right (void) {
	//std::cerr << __PRETTY_FUNCTION__ << std::endl;
	glRotated(90, 0, 1, 0);
}
void back (void) {
	//std::cerr << __PRETTY_FUNCTION__ << std::endl;
	glRotated(180, 0, -1, 0);
}


void glutDisplay (void) {
	const GLfloat light_pos[4] = {10.0,16.0,0.0,0.0};
	const GLfloat colors[3][4] = {  { 1.0, 0.0, 0.0, 1.0 },
					{ 0.0, 1.0, 0.0, 1.0 },
					{ 0.0, 0.0, 1.0, 1.0 } };
	GLfloat white[4] = {1., 1., 1., 1.};
	int i = 0, j;

	if (!event->isEmpty())	
	   {
	   *pos=event->getEvent();
           pos->sendUpdate();
  	   }   

	/* send updates, synchronize and update variables. All nodes do the
	update, so no matter who quits the application survives. */
        // pos == 0 to left   pos==1 to right
        // pos == 2 stop      pos==3 start
        if ((pos->getData()==0) || (pos->getData()==3))
           {
	   *deg = **deg + 1;
	   if (**deg > 360)
		*deg = 0;
           }
        else if (pos->getData()==1)
           {
           *deg=**deg-1;
           if (**deg<0)
              *deg=360;
           }
	else if (pos->getData()==4){
		((GlassClient *)g)->killAll();	
	}
	deg->sendUpdate();
	datalock->sync();
	deg->getUpdate();
        pos->getUpdate();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	view_set->getData()();	
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);

	/* draw the balls */
	for (i = 60; i >= -60; i -= 20 ) 
		for ( j = 0; j < 3; j++ )  {
			glPushMatrix();
			/* apply the rotation */
			glRotated((deg->getData()+j*120)%360, 0, 1, 0);

			glTranslatef(0, i, -50);

			glMaterialfv(GL_FRONT, GL_SPECULAR, white);
			glMaterialf(GL_FRONT, GL_SHININESS, 20.);
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colors[j]);
			glutSolidSphere(8.0, 10, 10);
			glPopMatrix();
		}

	swaplock->sync();
	glutSwapBuffers();
}


void glutIdle (void) {
	static int start = 0;
	int end;

#define BENCHMARK
#ifdef BENCHMARK
	if (deg->getData() == 0) {
		end = glutGet(GLUT_ELAPSED_TIME);
		printf("%d ms, %f fps\n", end-start, 360000.0/(float)(end-start));
		start = end;
	}
#endif
	glutPostRedisplay();
}


void glutResize (int w, int h) {
	winW = w;
	winH = h;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, winW, winH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90, winW / winH, 1, 9999);

	glutPostRedisplay();
}

void glutKeyboard (unsigned char key, int x, int y) {
	switch (key) {
		case '3':
		case '4':
		case '5':
		case '6':
			node_set = key - '3' + 3;
			break;
		case 'f':
		case 'F':
		        view_set->associate(node_set, "front");
			glutResize(winW, winH);
			break;
		case 'r':
		case 'R':
		        view_set->associate(node_set, "right");
			glutResize(winW, winH);
			break;
		case 'l':
		case 'L':
		        view_set->associate(node_set, "left");
			glutResize(winW, winH);
			break;
		case 'b':
		case 'B':
		        view_set->associate(node_set, "back");
			glutResize(winW, winH);
			break;
		case 'q':
		case 'Q':
			delete g;
			exit(1);
		default:
			break;
	}
}


int main (int argc, char **argv) { 

	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		std::cerr << e.getMessage() << std::endl;
		exit(1);
	}

	/* create a barrier for swaplock */
	datalock = new Barrier(1);
	swaplock = new Barrier(2);
        /* create our shared data, and set its value */
        deg = new Shared<int, rawPack, rawUnpack>("deg");
                                                                                 
        /* create our event data */
        event = new Event<int, rawPack, rawUnpack>("event");
                                                                                 
        /* pos ==0 turn to right  pos==1 turn to left  */
        pos = new Shared<int, rawPack, rawUnpack>("pos");


	if (g->getNodeId() == 3) {
		*deg = 0;
                *pos = 0;
		deg->sendUpdate();
                pos->sendUpdate();
	}

	/* create and set our view set function alias */
	view_set = new Alias<void (*)(void)>("view_set", front);
	view_set->addAlias((const char *)"front", front);
	view_set->addAlias((const char *)"right", right);
	view_set->addAlias((const char *)"left", left);
	view_set->addAlias((const char *)"back", back);

	if (((GlassClient *)g)->isMaster()) {
#ifdef TODOALIAS
	        view_set->associate(firstId+1, "front");
        	view_set->associate(firstId+2, "right");
	        view_set->associate(firstId+3, "left");
        	view_set->associate(firstId+4, "back");
#endif
	}
	
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(winW, winH);
	glutCreateWindow("Around - libGlass Application");
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutResize);
	glutDisplayFunc(glutDisplay);
	glutKeyboardFunc(glutKeyboard);

	/* setup environment */
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_DEPTH_TEST);
//	glDisable(GL_DITHER);
	glEnable(GL_CULL_FACE);
//	glCullFace(GL_BACK);
//	glFrontFace(GL_CCW);

	/* Set up light0 */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
//	glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
//	glLightfv(GL_LIGHT0, GL_SPECULAR, white);

	/* ambient and diffuse will track glColor */
/*	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	glMaterialfv(GL_FRONT, GL_SPECULAR, white);
	glMaterialf(GL_FRONT, GL_SHININESS, 20.); */

	glutMainLoop();
}
