/*  Glass - a distributed computing library
    Copyright (C) 2003-2010 Bruno Barberi Gnecco

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**********************************************
Multiprojection example

Program description:
  aroundcontrol.java : it is a java interface which control  a simple OpenGL program 
  demonstrating the integration of libGlass   wih any java application.
  
  (aroundcontrol.java was NOT gererated by the GEditor)
    
Dependencies:

Authors:
Marcelo de Paiva Guimaraes <marcelodepaiva@gmail.com>

History:
22 June  2003: Initial implementation (MPG)
30 March 2010: added JNA (MPG)

Return codes:
0       Successful

***************************************************************************/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.jna.Native;

public class aroundcontrol extends JFrame {
	JPanel jPanel1 = new JPanel();
	JButton jBleft = new JButton();
	JButton jBright = new JButton();
	JButton jBstop = new JButton();
	JLabel jLabel1 = new JLabel();
	JLabel jLabel2 = new JLabel();
	JButton jBstart = new JButton();
	JButton jBexit = new JButton();
	int	event;	
	Integration integ = (Integration) Native.loadLibrary("glassintegration", Integration.class);
	
	public aroundcontrol(String s){
		try{		    	
			jbInit();
		}catch(Exception e){
			e.printStackTrace();  
		}
		integ.clickevent(s);		
	}
	
	public void finalize(){
		integ.destroyevent();
	}
	
	public static void main(String args[]){
		if (args.length<1){
			System.out.println("\n   Use: java arounfinal  host");
			System.exit(0);
		}
		JFrame  frame = new aroundcontrol(args[0]);
		frame.pack();
		frame.setSize(new Dimension(300,350));
		frame.setVisible(true);
	}
	
	private void jbInit() throws Exception {
		this.getContentPane().setLayout(null);
		jPanel1.setBorder(BorderFactory.createEtchedBorder());
		jPanel1.setBounds(new Rectangle(60, 43, 156, 214));
		jPanel1.setLayout(null);
		jBleft.setBounds(new Rectangle(31, 13, 87, 32));
		jBleft.setText("Left");
		jBleft.addActionListener(new aroundcontrol_jBleft_actionAdapter(this));
		jBright.setBounds(new Rectangle(31, 60, 88, 30));
		jBright.setText("Right");
		jBright.addActionListener(new aroundcontrol_jBright_actionAdapter(this));
		jBstop.setBounds(new Rectangle(31, 104, 87, 33));
		jBstop.setText("Stop");
		jBstop.addActionListener(new aroundcontrol_jBstop_actionAdapter(this));
		jLabel1.setFont(new java.awt.Font("Dialog", 0, 15));
		jLabel1.setForeground(Color.blue);
		jLabel1.setText("Around - aroundcontrol");
		jLabel1.setBounds(new Rectangle(80, 12, 191, 25));
		jLabel2.setText("libGlass Application");
		jLabel2.setBounds(new Rectangle(131, 278, 196, 21));
		jBstart.setBounds(new Rectangle(32, 142, 88, 29));
		jBstart.setText("Start");
		jBstart.addActionListener(new aroundcontrol_jBstart_actionAdapter(this));
		jBexit.setBounds(new Rectangle(31, 175, 88, 29));
		jBexit.setText("Exit");
		jBexit.addActionListener(new aroundcontrol_jBexit_actionAdapter(this));  
		this.getContentPane().add(jPanel1, null);
		jPanel1.add(jBleft, null);
		jPanel1.add(jBright, null);
		jPanel1.add(jBstop, null);
		jPanel1.add(jBstart, null);
		jPanel1.add(jBexit, null);
		this.getContentPane().add(jLabel1, null);
		this.getContentPane().add(jLabel2, null);
	}
	void jBleft_actionPerformed(ActionEvent e) {
		integ.registerEvent(0); // left
	}
	void jBright_actionPerformed(ActionEvent e) {
		integ.registerEvent(1); //  right
	}
	void jBstop_actionPerformed(ActionEvent e) {
		integ.registerEvent(2); //stop  
	}
	void jBstart_actionPerformed(ActionEvent e) {
		integ.registerEvent(3); //  re-start
	}
	void jBexit_actionPerformed(ActionEvent e) {
		integ.registerEvent(4); //exit  
		integ.destroyevent();
		System.exit(0);
	}
}

class aroundcontrol_jBleft_actionAdapter implements java.awt.event.ActionListener {
	aroundcontrol adaptee;
	aroundcontrol_jBleft_actionAdapter(aroundcontrol adaptee) {
		this.adaptee = adaptee;
	}
	
	public void actionPerformed(ActionEvent e) {
		adaptee.jBleft_actionPerformed(e);
	}
}

class aroundcontrol_jBright_actionAdapter implements java.awt.event.ActionListener {
	aroundcontrol adaptee;
	
	aroundcontrol_jBright_actionAdapter(aroundcontrol adaptee) {
		this.adaptee = adaptee;
	}
	
	public void actionPerformed(ActionEvent e) {
		adaptee.jBright_actionPerformed(e);
	}
}

class aroundcontrol_jBstop_actionAdapter implements java.awt.event.ActionListener {
	aroundcontrol adaptee;
	
	aroundcontrol_jBstop_actionAdapter(aroundcontrol adaptee) {
		this.adaptee = adaptee;
	}
	
	public void actionPerformed(ActionEvent e) {
		adaptee.jBstop_actionPerformed(e);
	}
}

class aroundcontrol_jBstart_actionAdapter implements java.awt.event.ActionListener {
	aroundcontrol adaptee;
	
	aroundcontrol_jBstart_actionAdapter(aroundcontrol adaptee) {
		this.adaptee = adaptee;
	}
	public void actionPerformed(ActionEvent e) {
		adaptee.jBstart_actionPerformed(e);
	}
}
class aroundcontrol_jBexit_actionAdapter implements java.awt.event.ActionListener {
	aroundcontrol adaptee;
	
	aroundcontrol_jBexit_actionAdapter(aroundcontrol adaptee) {
		this.adaptee = adaptee;
	}
	public void actionPerformed(ActionEvent e) {
		adaptee.jBexit_actionPerformed(e);
	}
}
