#include "libglass.h"
#include <iostream>
#include <stdio.h>
#include <GL/glut.h>

using namespace libglass;

//Glass		*g;
GlassClient *g=NULL;
Barrier 	*datalock;
Barrier 	*swaplock;
Shared<int, rawPack, rawUnpack> *deg;
Alias<void(*)(void)> *alias;
nodeId		node_set=1;



int winW = 320, winH = 240;

void front (void) {

}
void left (void) {
	glRotated(90, 0, -1, 0);
}
void right (void) {
	glRotated(90, 0, 1, 0);
}
void back (void) {
	glRotated(180, 0, -1, 0);
}

void set_view() {
	int qtd = 0;
	std::map<nodeId, nodeNickName>::iterator i;
	nodeId targetid;
	if (g->isMaster()) {
		std::map<nodeId, nodeNickName> clients(g->getClientList());
		qtd = 1;
		for (i = clients.begin(); i != clients.end(); i++, qtd = (qtd + 1) % 4) {
			targetid = i->first;
			if (qtd == 1) {
				alias->associate(targetid, "front");
			} else if (qtd == 2) {
				alias->associate(targetid, "left");
			} else if (qtd == 3) {
				alias->associate(targetid, "right");
			} else {
				alias->associate(targetid, "back");
			}
		}
	}
	//printf("node  = %d \n",g->getNodeId());
	alias->getData()();

}

void glutDisplay (void) {
	const GLfloat light_pos[4] = {10.0,16.0,0.0,0.0};
	const GLfloat colors[3][4] = {  { 1.0, 0.0, 0.0, 1.0 },
					{ 0.0, 1.0, 0.0, 1.0 },
					{ 0.0, 0.0, 1.0, 1.0 } };
	GLfloat white[4] = {1., 1., 1., 1.};
	int i = 0, j;

	/* send updates, synchronize and update variables. All nodes do the
	update, so no matter who quits the application survives. */

	*deg = **deg + 1;
	if (**deg > 360)
		*deg = 0;
	deg->sendUpdate();
	datalock->sync();
	deg->getUpdate();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//view_set->getData()();
	set_view();
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);

	/* draw the balls */
	for (i = 60; i >= -60; i -= 20 ) 
		for ( j = 0; j < 3; j++ )  {
			glPushMatrix();
			/* apply the rotation */
			glRotated((deg->getData()+j*120)%360, 0, 1, 0);

			glTranslatef(0, i, -50);

			glMaterialfv(GL_FRONT, GL_SPECULAR, white);
			glMaterialf(GL_FRONT, GL_SHININESS, 20.);
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colors[j]);
			glutSolidSphere(8.0, 10.0, 10.0);
			glPopMatrix();
		}

	swaplock->sync();
	glutSwapBuffers();
}


void glutIdle (void) {
	static int start = 0;
	int end;

#define BENCHMARK
#ifdef BENCHMARK
	if (deg->getData() == 0) {
		end = glutGet(GLUT_ELAPSED_TIME);
		printf("%d ms, %f fps\n", end-start, 360000.0/(float)(end-start));
		start = end;
	}
#endif
	glutPostRedisplay();
}


void glutResize (int w, int h) {
	winW = w;
	winH = h;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, winW, winH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90, winW / winH, 1, 9999);

	glutPostRedisplay();
}

void glutKeyboard (unsigned char key, int x, int y) {
	std::map<nodeId, nodeNickName> clients(g->getClientList());
	std::map<nodeId, nodeNickName>::iterator i;

	i = clients.begin();
	switch (key) {
		case '1':
			node_set = i->first;
			printf("1   ");
			break;
		case '2':
			printf("2   ");
			i++;
			node_set = i->first;
			break;
		case '3':
			printf("3   ");
			i++;i++;
			node_set = i->first;
			break;
		case '4':
			printf("4   ");
			i++;i++;i++;
		    node_set = i->first;
			break;
		case 'f':
		case 'F':
			printf("node %d   front \n",node_set);
		    alias->associate(node_set, "front");
			glutResize(winW, winH);
			break;
		case 'r':
		case 'R':
			printf("node %d   right \n",node_set);
			alias->associate(node_set, "right");
			glutResize(winW, winH);
			break;
		case 'l':
		case 'L':
			printf("node %d   left \n",node_set);
		    alias->associate(node_set, "left");
			glutResize(winW, winH);
			break;
		case 'b':
		case 'B':
			printf("node %d   back \n",node_set);
		    alias->associate(node_set, "back");
			glutResize(winW, winH);
			break;
		case 'q':
		case 'Q':
			g->killAll();
			delete g;
			exit(1);
		default:
			break;
	}
}


int main (int argc, char **argv) { 

	try {
		if (argc == 1) {
			new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		std::cerr << e.getMessage() << std::endl;
		exit(1);
	}

	/* create a barrier for swaplock */
	datalock = new Barrier(1,false);
	swaplock = new Barrier(2,false);

	/* create our shared data, and set its value */
	deg = new Shared<int, rawPack, rawUnpack>("deg");
	//if (g->getNodeId() == 3) {
	if (g->isMaster()) {
		*deg = 0;
		deg->sendUpdate();
	}

	/* create and set our view set function alias */
	alias = new Alias<void(*)(void)> ("integer", front);
	alias->addAlias((const char *) "front", front);
	alias->addAlias((const char *) "right", right);
	alias->addAlias((const char *) "left", left);
	alias->addAlias((const char *) "back", back);

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(winW, winH);
	glutCreateWindow("Around - Glass Application");
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutResize);
	glutDisplayFunc(glutDisplay);
	glutKeyboardFunc(glutKeyboard);

	/* setup environment */
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_DEPTH_TEST);
//	glDisable(GL_DITHER);
	glEnable(GL_CULL_FACE);
//	glCullFace(GL_BACK);
//	glFrontFace(GL_CCW);

	/* Set up light0 */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
//	glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
//	glLightfv(GL_LIGHT0, GL_SPECULAR, white);

	/* ambient and diffuse will track glColor */
/*	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	glMaterialfv(GL_FRONT, GL_SPECULAR, white);
	glMaterialf(GL_FRONT, GL_SHININESS, 20.); */

	glutMainLoop();
}
