/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
  Cubes: a simple OpenGL program demonstrating the use of libGlass for
  distributed multidisplay rendering.
*/

#ifndef BROKEN

#include <stdio.h>
#include <math.h>
#include <iostream>
#ifdef MACOSX
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
 #include <GLUT/glut.h>
#else
 #include <GL/glut.h>
#endif
#include "libglass.h"

#ifndef M_PI
#define M_PI 3.14159265
#endif

using namespace libglass;
using std::endl;

struct Data {
	char		stereo;
	char		fullscreen;
	int		width, height;
	float		eyesep;
	unsigned int	cubes[3];
	char		anim;
};

struct Cube {
	GLfloat		rot[3];
};

struct Sphere {
	GLdouble	radius;
	GLint		slices,stacks;
};

static struct Data data = { 0, 0, 800, 600, 0.1, { 8, 8, 8 }, 0 };

static struct Cube *cubes;

static struct Sphere sphere = { 0.0, 14, 10 };

static float vel[4] = { 0.03, 0.0033, 0.0027, 0.0030 };

Glass		*g;
Barrier 	*datalock, *swaplock;
Alias<void (*)(void)> *view_set;
Shared<float, rawPack, rawUnpack> *pos0,*pos1,*pos2,*pos3;



static GLfloat lightpos[4] = { 5.0, 5.0, 10.0, 0.0 };

void front(void)
{
	std::cerr << __func__ << std::endl;
}
void left(void)
{
	std::cerr << __func__ << std::endl;
	glRotated(90, 0, -1, 0);
}
void right(void)
{
	std::cerr << __func__ << std::endl;
	glRotated(90, 0, 1, 0);
}
void back(void)
{
	std::cerr << __func__ << std::endl;
	glRotated(180, 0, -1, 0);
}
void floor(void)
{
	std::cerr << __func__ << std::endl;
        glRotated(90, 1, 0, 0);
}


static void render(void) {
	unsigned int i;

	glTranslatef(0, 0, pos0->getData());
	glRotatef(pos1->getData(), 0.0, 0.0, 1.0);
	glRotatef(pos2->getData(), 1.0, 0.0, 0.0);
	glRotatef(pos3->getData(), 0.0, 1.0, 0.0);
	glColor4f(1.0, 0.0, 0.0, 1.0);
	glFrontFace(GL_CW);
	glutSolidSphere(sphere.radius, sphere.slices, sphere.stacks);
	glFrontFace(GL_CCW);
	glColor4f(1.0, 0.0, 0.0, 1.0);
	for (i = 1; i <= data.cubes[0]*data.cubes[1]*data.cubes[2]; i++) {
		glPushMatrix();
		glTranslatef(4*((float)(i%data.cubes[0])-(float)(data.cubes[0]/2)),
			4*((float)((i/data.cubes[0])%(data.cubes[1]))-(float)(data.cubes[1]/2)),
			4*((float)((i/data.cubes[0]/data.cubes[1])%(data.cubes[0]))-(float)(data.cubes[2]/2)));
		glRotatef(cubes[i-1].rot[0], 1.0, 0.0, 0.0);
		glRotatef(cubes[i-1].rot[1], 0.0, 1.0, 0.0);
		glRotatef(cubes[i-1].rot[2], 0.0, 0.0, 1.0);
		glutSolidCube(1.0);
		glPopMatrix();
	}
}

static void framelock() {

	pos0->sendUpdate();
	pos1->sendUpdate();
	pos2->sendUpdate();
	pos3->sendUpdate();
	datalock->sync();
	pos0->getUpdate();
	pos1->getUpdate();
	pos2->getUpdate();
	pos3->getUpdate();
}

GLfloat t = 0;
static void draw(void) {
	static GLint T0 = 0;
	static GLint Frames = 0;

	framelock();

	glLoadIdentity();

	if (data.stereo) {
		glDrawBuffer(GL_BACK_RIGHT);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glPushMatrix();
		glTranslatef(data.eyesep, 0.0, 0.0);
		view_set->getData()();
		glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
		render();
		glPopMatrix();
		glDrawBuffer(GL_BACK_LEFT);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glPushMatrix();
		glTranslatef(-data.eyesep, 0.0, 0.0);
		view_set->getData()();
		glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
		render();
		glPopMatrix();
	}
	else {
		render();
	}
	swaplock->sync();
	glutSwapBuffers();

	Frames++;
	{
		GLint t1 = glutGet(GLUT_ELAPSED_TIME);
		if (t1 - T0 >= 5000) {
			GLfloat seconds = (t1 - T0) / 1000.0;
			GLfloat fps = Frames / seconds;
			printf("%d frames in %g seconds = %g FPS\n", Frames, seconds, fps);
			T0 = t1;
			Frames = 0;
		}
	}
	glutPostRedisplay();
}

static void idle(void) {
	if (data.anim) {
		t += 0.3;
		*pos0 = 1.4*sin(vel[0]*t)+1.42;
		*pos1 = -360.0*sin(vel[1]*t);
		*pos2 = -360.0*sin(vel[2]*t);
		*pos3 =  360.0*sin(vel[3]*t);
	}

	glutPostRedisplay();
}

/* change view angle, exit upon ESC */
static void key(unsigned char k, int x, int y)
{
	switch (k) {
	  case ' ':
		  data.anim = !data.anim;
		  break;
	  case 27:		/* Escape */
	  case 'q':
		  exit(0);
		  break;
	  case 'a':
		  *pos0 = pos0->getData() + 0.005;
		  break;
	  case 'z':
		  *pos0 = pos0->getData() - 0.005;
		  break;
	  case 's':
		  *pos1 = pos1->getData() + 0.0001;
		  break;
	  case 'x':
		  *pos1 = pos1->getData() - 0.0001;
		  break;
	  case 'd':
		  *pos2 = pos2->getData() + 0.0001;
		  break;
	  case 'c':
		  *pos2 = pos2->getData() - 0.0001;
		  break;
	  case 'f':
		  *pos3 = pos3->getData() - 0.0001;
		  break;
	  case 'v':
		  *pos3 = pos3->getData() - 0.0001;
		  break;
	  case 'p':
//		  printf("[%f %f %f %f]\n", vel[0], vel[1], vel[2], vel[3]);
		  break;
	  default:
		  break;
	}
	glutPostRedisplay();
}

/* change view angle */
static void special(int k, int x, int y)
{
	switch (k) {
	  default:
		  return;
	}
	glutPostRedisplay();
}

/* new window size or exposure */
static void reshape(int width, int height)
{
	GLfloat h = (GLfloat) height / (GLfloat) width;

	glViewport(0, 0, (GLint) width, (GLint) height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
//	glFrustum(-1.0, 1.0, -h, h, 5.0, 60.0);
	gluPerspective(90, h, 0.1, sphere.radius+0.5);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glutPostRedisplay();
}

static void initgl() {
	//const static GLfloat red[4] = { 0.8, 0.1, 0.0, 1.0 };
	//const static GLfloat green[4] = { 0.0, 0.8, 0.2, 1.0 };
	//const static GLfloat blue[4] = { 0.2, 0.2, 1.0, 1.0 };
	unsigned int i;

	/* ok, generate cubes */
	cubes = (struct Cube *)malloc(data.cubes[0]*data.cubes[1]*data.cubes[2]
			*sizeof(struct Cube));
	if (!cubes) {
		exit(1);
	}

//	srand(time(0));
	srand(3227);
	for (i = 0; i < data.cubes[0]*data.cubes[1]*data.cubes[2]; i++) {
		cubes[i].rot[0] = (float)(rand()%360);
		cubes[i].rot[1] = (float)(rand()%360);
		cubes[i].rot[2] = (float)(rand()%360);
	}

	/* bounding sphere */
	i = (data.cubes[0] > data.cubes[1] ? data.cubes[0] : data.cubes[1]);
	i = (i > data.cubes[2] ? i : data.cubes[2]);
	sphere.radius = i*4*2;

	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
        glEnable(GL_COLOR_MATERIAL);
        glMaterialf(GL_FRONT, GL_SHININESS, 20.);
}

void visible(int vis)
{
	if (vis == GLUT_VISIBLE)
		glutIdleFunc(idle);
	else
		glutIdleFunc(NULL);
}

static int args (int argc, char **argv) {
	int i = 1;
	while (i < argc) {
		if (*argv[i] != '-') {
			return i;
		}
		switch (*(argv[i]+1)) {
		   case 's':
			data.stereo = 1;
			break;
		   case 'f':
			data.fullscreen = 1;
			break;
		   case 'w':
			data.width = atoi(argv[++i]);
			break;
		   case 'h':
			data.height = atoi(argv[++i]);
			break;
		   case 'e':
			data.eyesep = atof(argv[++i]);
			break;
/*		   case 'c':
			sscanf(argv[++i], "%d %d %d", &data.cubes[0],
					&data.cubes[1], &data.cubes[2]);
			break; */
		   default:
			std::cout <<"Arguments are:" << endl <<
				"-s\tstereo" << endl <<
				"-f\tfullscreen" << endl <<
				"-w [n]\twidth" << endl <<
				"-h [n]\theight" << endl <<
				"-e [n]\teye separation (parallax)" << endl <<
				"-n [x y z]\tnumber of cubes" << endl;
			exit(1);
			break;
		}
		i++;
	}
	return 0;
}

int main (int argc, char *argv[]) {
	glutInit(&argc, argv);

	int server = args(argc, argv);

	try {
		if (!server) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[server]);
		}
	}
	catch (Exception e) {
		std::cerr << e.getMessage() << std::endl;
		exit(1);
	}

	/* Create variables. */
	datalock = new Barrier(1,false);
	swaplock = new Barrier(2,false);

	/* create our shared data, and set its value */
	pos0 = new Shared<float, rawPack, rawUnpack>("pos0");
	pos1 = new Shared<float, rawPack, rawUnpack>("pos1");
	pos2 = new Shared<float, rawPack, rawUnpack>("pos2");
	pos3 = new Shared<float, rawPack, rawUnpack>("pos3");

	/* create and set our view set function alias */
	view_set = new Alias<void (*)(void)>("view_set", front);
	view_set->addAlias((const char *)"front", front);
	view_set->addAlias((const char *)"right", right);
	view_set->addAlias((const char *)"left", left);
	view_set->addAlias((const char *)"back", back);
	view_set->addAlias((const char *)"floor", floor);

	if (((GlassClient *)g)->isMaster()) {
#ifdef TODOALIAS
			view_set->associate(firstId+1, "front");
        	view_set->associate(firstId+2, "right");
	        view_set->associate(firstId+3, "left");
        	view_set->associate(firstId+4, "back");
        	view_set->associate(firstId+5, "floor");
#endif
		*pos0 = 0.03;		pos0->sendUpdate();
		*pos1 = 0.0033;		pos1->sendUpdate();
		*pos2 = 0.0027;		pos2->sendUpdate();
		*pos3 = 0.0030;		pos3->sendUpdate();
	}

	if (data.stereo)
		glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE | GLUT_STEREO);
	else
		glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);

	if (data.fullscreen) {
		char buf[100];
		sprintf(buf, "%dx%d:32", data.width, data.height);
		glutGameModeString(buf);
		glutEnterGameMode();
		glutSetCursor(GLUT_CURSOR_NONE);
	}
	else {
		glutInitWindowPosition(0, 0);
		glutInitWindowSize(300, 300);
		glutCreateWindow("Cubes by BBG");
	}
	initgl();
	glutDisplayFunc(draw);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutSpecialFunc(special);
	glutVisibilityFunc(visible);

	glutMainLoop();
	return 0;		/* ANSI C requires main to return int. */
}
#else
int main() {
	return 0;
}

#endif //BROKEN
