/************************************************************************
 *             Parallel Quicksort algorithm                             *
 *  It can use 1, 2 or 4 slaves nodes                                   *
 *  autors                                                              *
 *          Marcelo de Paiva Guimaraes , Bruno B. Gnecco                *
 *                                                                      *
 ************************************************************************/

#include <iostream>
#include <stdlib.h>
#include <string>
#include  <pthread.h>

#define MAXQUICK 100                 //Vector size
#ifdef UNIX
 #include <sys/times.h>
#endif

#include "libglass.h"

using namespace libglass;
using std::cerr;
using std::endl;

struct _time {
#ifdef UNIX
	struct timeval		time;
#endif
};
typedef struct _time Time;


void Exchange(int&,int&);
void QuickSort(int[],int,int);
void noneSlave();
void oneSlave();
void twoSlave();
void printElements();
void sendElements(int node, int start, int end);
void receiveElements(int node, int quant, int pos);
void generateElements();
int Partition(int A[],int l,int r);
void getTime(Time &t);
void printTime(Time t); 
Time elapsedTime(Time &x, Time &y);


int *A, l=0,r=MAXQUICK-1,i;
string s;
Time   start,end;

Glass		*g;
Messenger	*m, *n;
Shared<int> *step, *half;
pthread_t      th_1,th_2;
int l1,r1,jj;
	
int main(int argc, char **argv) {
	A = new int[MAXQUICK];
	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}
	try {
		m = new Messenger("messenger");
		n = new Messenger("messenger2");
	}
	catch(Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}
	step = new Shared<int>("step");
	half = new Shared<int>("half");
        *step=0;
	step->sendUpdate();
	if (g->getNodeId()==6)
		{
		generateElements();	
		printElements();		
		}
	getTime(start);
	//noneSlave();
        //oneSlave();
	twoSlave();
	getTime(end);
}



void noneSlave()
	{
//	generateElements();
	QuickSort(A,l,r);
	printElements();	
	}

void oneSlave()
	{
	if (g->getNodeId()==6) // master
		{
		// wait the slave connection
		while (step->getData()==0) 
			{step->getUpdate();}
//		generateElements();	
		sendElements(7,l,r);	
		// notify the sending process finished
		*step=2;
		step->sendUpdate();
		step->getUpdate();
		while (step->getData()==2) 
			{step->getUpdate();}
		receiveElements(7,MAXQUICK,0);	
		printElements();
		}
	else
		{
		 // notify that the slave is connected
		*step=1;
		step->sendUpdate();
		step->getUpdate();
		// receive the elements
		receiveElements(6,MAXQUICK-1,0);
		//call the ordering function
		QuickSort(A,l,r);
		*step=3;
		step->sendUpdate();
		sendElements(6,0,MAXQUICK);			
		//printElements();
		}		
	}

void* thread_send(void *arg)
	{
	sendElements(8,jj+1,r+1);
	}
void* thread_receive(void *arg)
	{
	int xquant=0;
	int quant=MAXQUICK-half->getData()-1;
	int pos=half->getData()+1;
	int node =8;
	string s;
	while (xquant<quant)
		{
		step->getUpdate();
		if (n->getMessage(node, s))
			{
			std::stringstream is(s);        
			is >> A[pos];
			pos++;
			xquant++;
			}
		}	
	}
void* thread_receive2(void *arg)
	{
	receiveElements(7,half->getData(),0);
	}
void twoSlave()
	{
	if (g->getNodeId()==6) // master
		{
		// wait the slave connection
//		generateElements();	
		//printElements();
		while (step->getData()!=2) 
			{step->getUpdate();}
		sendElements(7,l,r+1);	
		// notify the sending process finished
		*step=2;
		step->sendUpdate();
		step->getUpdate();
		while (step->getData()==2) 
			{step->getUpdate();}
		// todo : thread para slave 1 e outra para slave 2	
		half->getUpdate();
		pthread_create(&th_1, NULL, thread_receive, NULL);
		pthread_create(&th_2, NULL, thread_receive2, NULL);
		//receiveElements(7,half->getData(),0);	
		pthread_join(th_1, NULL);
		pthread_join(th_2, NULL);
		printElements();
		}
	else if(g->getNodeId()==7) // slave 1
		{
		 // notify that the slave is connected
		*half=0;
 	        half->sendUpdate();
		*step=1;
		step->sendUpdate();
		step->getUpdate();
		// receive the elements
		receiveElements(6,MAXQUICK-1,0);
 	        jj = Partition(A,l,r);
		*half=jj;
		half->sendUpdate();
		half->getUpdate();
		l1=l;
		r1=r;
		while (step->getData()!=2)
			{
			step->getUpdate();
			}
		pthread_create(&th_1, NULL, thread_send, NULL);
//		sendElements(8,jj+1,r+1);	
		QuickSort(A,l1,jj-1);	
		//call the ordering function
		*step=3;
		step->sendUpdate();
		sendElements(6,l1,jj+1);
		
		}	
	else if(g->getNodeId()==8) // slave 2
		{
		 // notify that the slave is connected
		*step=2;
		step->sendUpdate();
		step->getUpdate();
		// receive the elements
		while (half->getData()==0)
			{
			half->getUpdate();
			}
		receiveElements(7,MAXQUICK-half->getData()-1,half->getData()+1);
		i=0;
		QuickSort(A,half->getData(),MAXQUICK-1);
		i=half->getData()+1;
		i=0;
		//call the ordering function
		*step=3;
		step->sendUpdate();
		half->getUpdate();
		int start=half->getData()+1;
		int end=MAXQUICK;		
		for(i=start;i<end;i++)
			{
			std::stringstream ss;
		        std:: string str;
			ss << A[i];
			ss >> str;
			n->sendMessage(6,str);
			}
		}	
	}	
	
void generateElements( )
	{
	for(i=0;i<MAXQUICK;i++)
		{
		A[i]= rand()%100;
		}
	}

void sendElements(int node, int start, int end)
	{
	for(i=start;i<end;i++)
		{
		std::stringstream ss;
	        std:: string str;
		ss << A[i];
		ss >> str;
		m->sendMessage(node,str);
		}
	}
	
void receiveElements(int node, int quant, int pos)
	{
	int xquant=0;
	while (xquant<quant)
		{
		step->getUpdate();
		if (m->getMessage(node, s))
			{
			std::stringstream is(s);        
			is >> A[pos];
			pos++;
			xquant++;
			}
		}	
	}
void printElements()
	{
	// print the vector ordened
	cout<<"\nOrdened Sequence: \n";
	for(i=0;i<MAXQUICK;i++)
		cout<< "  "<<A[i];
	cout<<"\nTime: ";	
	printTime(elapsedTime(end, start));
	}

//Exchange the value of 2 variables
void Exchange(int &a,int & b)
{
	int aux = a;
	a = b;
	b = aux;
}

//Receives the segment A, the left(l) and right (r) extremity
//returns j and A, where l<=j<=r and A is ordened accordily
//A[l],...A[j-1] sao menores que A[j],...,A[k]
int Partition(int A[],int l,int r)
{
	int j,k;
	j = l;
	k = r;
	while( j<k )
	{
		while( j<k && A[j]<=A[k] ) k--;
		if( j<k )
			Exchange( A[j++],A[k] );

		while( A[j]<A[k] ) j++;
		if( j<k )
			Exchange( A[j],A[k--] );
	}
	return j;
}

//Procedimento principal do algoritmo de ordenacao QuickSort
void QuickSort(int A[],int l,int r)
{
	int j;
	if( l<r )
	{
	  j = Partition(A,l,r);
	  QuickSort(A,l,j-1);
	  QuickSort(A,j+1,r);
	}
}

void getTime(Time &t) {
#ifdef UNIX
	gettimeofday(&t.time, NULL);
#endif
}

void printTime(Time t) {
#ifdef UNIX
	printf("%ld:%ld\n", t.time.tv_sec, t.time.tv_usec);
#endif
}

Time elapsedTime(Time &x, Time &y) {
#ifdef UNIX
	Time		result;

	/* Perform the carry for the later subtraction by updating Y. */
	if (x.time.tv_usec < y.time.tv_usec) {
		int nsec = (y.time.tv_usec - x.time.tv_usec) / 1000000 + 1;
		y.time.tv_usec -= 1000000 * nsec;
		y.time.tv_sec += nsec;
	}
	if (x.time.tv_usec - y.time.tv_usec > 1000000) {
		int nsec = (x.time.tv_usec - y.time.tv_usec) / 1000000;
		y.time.tv_usec += 1000000 * nsec;
		y.time.tv_sec -= nsec;
	}

	/* Compute the time remaining to wait.
	   `tv_usec' is certainly positive. */
	result.time.tv_sec = x.time.tv_sec - y.time.tv_sec;
	result.time.tv_usec = x.time.tv_usec - y.time.tv_usec;

	return result;
#endif
}
