/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2011 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_ADDRESS_HH_
#define _GLASS_ADDRESS_HH_

#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/un.h>

namespace libglass {

/*!
*/
// --- Temporarily disabled ---
// #ifdef _MSC_VER

// typedef unsigned int nodeFD;

// #  define GLASS_ERROR_FD INVALID_SOCKET
// #  define GLASS_SOCKET_ERROR  SOCKET_ERROR

// const nodeFD nobodyFD = ~0;
// const nodeFD everybodyFD = (~0)-1;

// #else

/*!
  Node File Descriptor.

  NodeFDs are used as an abstraction of file descriptors.
  */
typedef int nodeFD;
typedef std::string nodeNickName;

#  define GLASS_ERROR_FD (-1)
#  define GLASS_SOCKET_ERROR (-1)

const nodeFD nobodyFD = -1;
const nodeFD everybodyFD = -2;

class nodeAddress {
public:
	string			host; /// the hostname or ip
	uint16_t		port; /// the port
	nodeNickName	nickname;
	nodeRelation	relation;
	int				status; /// used for return values of the Protocol functions

	/*
	 * TODO: attributes below really belong to an extension of nodeAddress.
	 * We need a std::vector of nodeAddressExtensions
	 */
	nodeFD			fd;
	struct sockaddr_in		in_addr; /// the 32-bit IPV4 address
	struct sockaddr_in6		sockaddr_storage; /// the 16-byte IPV6 address
	struct sockaddr_un		un_addr; /// unix sockets

	/**
	 *
	 * @param host
	 * @return
	 */
	explicit nodeAddress(string host = "", uint16_t port = 0, nodeRelation relation = None);

	virtual ~nodeAddress();

	void setPort(uint16_t port);

	inline void setRelation(nodeRelation r) { this->relation = r; }

	/**
	 *
	 * @return true if this fd is valid.
	 */
	inline bool isValidFD(void) const { return fd >= 0; }

	/**
	 *
	 * @return true if there's a direct relation.
	 */
	inline bool isDirectRelation(void) const { return (relation != None && relation != Unregistered); }

	virtual bool operator < (const nodeAddress &addr) const;
	virtual bool operator == (const nodeAddress &addr) const;

	/*!
	  A nice outputter, for debugging purposes.
	*/
	virtual std::ostream& print(std::ostream &o) const;

	friend std::ostream& operator<<(std::ostream &o, const nodeAddress &p);
};

extern std::ostream& operator<<(std::ostream &o, const nodeAddress &p);

extern const nodeAddress nobodyAddress;
extern const nodeAddress everybodyAddress;

}

#endif
