/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_SERVER_HH_
#define _GLASS_SERVER_HH_

#include <list>

#include "glass.h"
#include "types.h"
#include "exception.h"
#include "thread.h"

namespace libglass {

class Protocol;

/*!
  Glass Server class.
*/
class LIBGLASS_API GlassServer : public Glass {
protected:
	nodeId			master; 	/**< current master id */

	/*!
	  GlassServerConnection is responsible for listening to new connections.
	*/
	class GlassServerConnection : public GlassThread {
	private:
		GlassServer		*s;	/**< our owner. */
	public:
		GlassServerConnection(GlassServer *s) throw(Exception);
		/*! Thread virtual function. */
		void cleanup(void);
		/*! Thread virtual function. */
		void execute(void);
	};
	friend class GlassServerConnection;

	GlassServerConnection	*sc;	/**< our server listener */

	/*!
	  Returns our protocol.
	  Used by GlassServerConnection.
	  \return The protocol.
	*/
	Protocol *getProtocol(void) const;

	/*!
	  Updates data relevant to clients (number, list)
	  \retval true
	  \retval false
	*/
	virtual bool updateClients(void);

	virtual bool rootPacket(Packet &p);

	virtual nodeId registerNode(nodeAddress &addr, nodeId id, nodeRelation r, bool force);
	virtual bool unregisterNode(nodeId id);

public:
	/*!
	  Constructor.
	  \param p The protocol to use.
	  \param autostart Should we start automatically? If false, you need to call start().
	  \throws Exception.
	*/
	explicit GlassServer(Protocol *p, bool autostart = true) throw(Exception);

	virtual ~GlassServer();

	/*!
	  Main loop.
	  \return
	*/
	virtual int start(void);

	void stop(void);
};

} // namespace


#endif // _GLASS_SERVER_HH_
