/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_SERVER_PEER_HH_
#define _GLASS_SERVER_PEER_HH_

#include "glassserver.h"
#include <set>

namespace libglass {

class Protocol;

/*!
  Glass Server class.
*/
class LIBGLASS_API GlassServerPeer : public GlassServer {
protected:
	std::string seedpeer; /**< The first peer we should connect to. */
	unsigned int seedpeerport; /**< The first peer port. */

	std::map<nodeId, nodeAddress> peerlist; /**< List of peers we are connected to. */

	virtual bool rootPacket(Packet &p);

	nodeId generateId(void) const;

public:
	/*!
	  Constructor.
	  \param p The protocol to use.
	  \throws Exception.
	*/
	explicit GlassServerPeer(Protocol *p, const char *peer = NULL, unsigned int peerport = 0) throw(Exception);
	virtual ~GlassServerPeer();

	virtual nodeId registerNode(nodeAddress &addr, nodeId id, nodeRelation r, bool force);
	virtual bool unregisterNode(nodeId id);
	virtual nodeAddress IdToAddress(nodeId id);


	/*!
	  Main loop.
	  \return
	*/
	virtual int start(void);

	/*!
	 Connects to a new peer. New peers are connected to automatically, but you may
	 wisth to join a whole new group.

	 \param peer The hostname or ip.
	 \return boolean
	 */
	bool connect(const char *peer);
};

} // namespace


#endif // _GLASS_SERVER_PEER_HH_
