/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_PROTOCOL_INTERPROCESS_HH_
#define _GLASS_PROTOCOL_INTERPROCESS_HH_

#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/ipc/message_queue.hpp>
#include <boost/interprocess/sync/scoped_lock.hpp>
#include <boost/interprocess/sync/named_mutex.hpp>

#include "memorymanager.h"

namespace libglass {

/**
 * Interprocess lock class. This is essentially a scoped lock on a named mutex.
 */
class InterprocessLock : public _Lock {
protected:
	boost::interprocess::named_mutex &mutex;
public:
	InterprocessLock(MemoryManager &mm, boost::interprocess::named_mutex &mutex);

	virtual ~InterprocessLock();

	/**
	 * Unlocks
	 */
	virtual void unlock(void);
};

/**
 * Structure to hold allocated adata
 */
class _MemData {
public:
	_MemData() : shm(NULL), region(NULL), length(0) {}
	~_MemData() {
		if (region) delete region;
		if (shm) delete shm;
	}

	boost::interprocess::shared_memory_object *shm;
	boost::interprocess::mapped_region *region;
	int length; /// total bytes allocated
};
typedef boost::shared_ptr<_MemData> MemData;

/**
 * Implements interprocess communication.
 *
 */
class LIBGLASS_API Interprocess : public MemoryManager {

public:
	Interprocess();

	virtual ~Interprocess();

	virtual void* allocate(size_t nbytes, const std::string &identifier = "");

	virtual void deallocate(void *p);

	virtual Lock scoped_lock();

};

} //namespace

#endif // _GLASS_PROTOCOL_INTERPROCESS_HH_
