/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_PROTOCOL_TCP_HH_
#define _GLASS_PROTOCOL_TCP_HH_

#include <list>

#define GLASS_TCP_DEFAULT_PORT 9191

#if defined(_MSC_VER)
# include <winsock2.h>
# include <time.h>
#else
# include <netinet/in.h>
# include <sys/time.h>
# include <sys/select.h>
# include <unistd.h>
#endif
#include <sys/types.h>
#include "protocol.h"
#include "packet.h"

namespace libglass {

class PacketLog;


#if defined(_MSC_VER)
// Instantiate std::list<T> template containers used by class Protocol
//LIBGLASS_IMP_TEMPLATE_STL_LIST(Packet*);
#endif // _MSC_VER

class LIBGLASS_API TCP : public Protocol {
protected:

	/**
	 * Binds socket
	 */
	virtual int bind(int listenfd);

	/**
	 * Reads packet
	 * @param p The packet to read into
	 * @param addr The address to read from
	 */
	virtual int readPacket(Packet &p, nodeAddress &addr);
	virtual unsigned int writePacket(const nodeAddress &addr, chain &m, unsigned int length);

	int PROTOCOL; /**< protocol name for socket() */

	std::list<Packet *>		queue;

	boost::mutex	mutex;		/**< Mutex for atomic operations */

	/**! Reentrant wrapper for gethostbyname

	 \return NULL if error, otherwise a string that must be freed with free().
	 */
	const struct in_addr gethostbyname_r(const char *host) throw(Exception);


	unsigned int listenport;

	int			listenfd;		/**< Listener socket. */
	int			fdmax;		/**< Maximum client socket number. */
	fd_set		socklist;	/**< List of sockets. */
	struct timeval	default_timeout;/**< Default timeout */
	bool		no_timeout;

	PacketLog	*plog;

	virtual int _listen(unsigned int port);
	virtual nodeAddress _accept(void);
	virtual nodeAddress _connect(const char *host, unsigned int port);
	virtual nodeAddress _connect(const nodeAddress &host);
	virtual void _disconnect(const nodeAddress &node);
	virtual bool _send(Packet &p);
	virtual int _recv(Packet &p);
	virtual int _safely_close_socket(nodeFD fd);

public:
	explicit TCP(unsigned int port = GLASS_TCP_DEFAULT_PORT);
	virtual ~TCP();

	nodeId getUniqueId(void);

	unsigned int getListenPort(void) {
		return this->listenport;
	}

	virtual nodeAddress getLocalAddress(void);
	virtual string getLocalHostname(void);

};

} //namespace

#endif // _GLASS_PROTOCOL_TCP_HH_
