/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_PROTOCOL_UDP_HH_
#define _GLASS_PROTOCOL_UDP_HH_

#include <map>

#if !defined(_MSC_VER)
#  include <netinet/in.h>
#endif

namespace libglass {

class LIBGLASS_API UDP : public Protocol {
private:
//	int readPacket(Packet &p, nodeAddress &addr);

	unsigned int writePacket(nodeAddress &addr, unsigned char *m,
			unsigned int length, struct sockaddr_in *addr);
 
protected:
	//! Default port.
#if defined(_MSC_VER) && (_MSC_VER < 1300)  // earlier than .NET compiler (VC 7.0)
	// Microsoft Knowledge Base Article - 241569 
	// Workaround for BUG: C2258 and C2252 on in Place Initialization of Static Const Members
	enum{ default_connect_port = 9192 };
	enum{ default_comm_port = 9193 };
#else
	static const unsigned int 	default_connect_port = 9192;
	static const unsigned int 	default_comm_port = 9193;
#endif

	unsigned int			comm_port;

	//! Internal data union (server/client).	
	struct server_struct { // Structure requires a name for std::map constructor to be generated under MSVC (see MSVC compiler error C2639)
		int		sockfd;		/**< Listener socket. */

		std::map<nodeAddress, struct nodedata *>
				clients;
	} server;
	struct client_struct { // Structure requires a name for std::map constructor to be generated under MSVC (see MSVC compiler error C2639)
		int		sockfd;		/**< Socket to server. */
		struct sockaddr_in 
				address;	/**< To connect to the server */
	} client;

	PacketLog			*plog;

	virtual int _listen(unsigned int port);
	virtual nodeAddress _accept(void);
	virtual nodeAddress _connect(const char *host, unsigned int port);
	virtual void _disconnect(nodeAddress &node);
	virtual bool _send(Packet &p);
	virtual int _recv(Packet &p);
public:
	UDP();
	virtual ~UDP();
//	bool setPort(unsigned int comm_port);
};


}// namespace

#endif // _GLASS_PROTOCOL_UDP_HH_
