/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "types.h"
#include <cstring>

namespace libglass {

nodeAddress::nodeAddress(string host, uint16_t port, nodeRelation relation) :
		host(host),
		relation(relation),
		status(0),
		fd(nobodyFD) {
	memset((void *)&in_addr, 0, sizeof(in_addr));
	memset((void *)&sockaddr_storage, 0, sizeof(sockaddr_storage));
	this->setPort(port);
}

nodeAddress::~nodeAddress() {

}

void nodeAddress::setPort(u_int16_t port) {
	this->port = port;
	in_addr.sin_port = port;
	sockaddr_storage.sin6_port = port;
}

bool nodeAddress::operator < (const nodeAddress &addr) const {
	if (addr.fd != this->fd)
		return this->fd < addr.fd;
	return host < addr.host; // TODO: this should work by now.
}

bool nodeAddress::operator == (const nodeAddress &addr) const {
	// TODO: do we need to check in_addr?
	if (fd == addr.fd &&
		host == addr.host &&
		nickname == addr.nickname) {
		return true;
	}
	return false;
}

/*!
  A nice outputter, for debugging purposes.
*/
std::ostream& nodeAddress::print(std::ostream &o) const {
	o << "Address host:" << this->host << "; FD=" << this->fd <<
			"; nickname=" << (this->nickname.size() > 0 ? this->nickname : "") <<
			"; relation=" << this->relation;
	return o;
}

std::ostream& operator<<(std::ostream &o, const nodeAddress &p) {
	return p.print(o);
}

const nodeAddress nobodyAddress("__nobody__", 0, Unregistered);
const nodeAddress everybodyAddress("__everybody__");

}
