/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "log.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#ifdef HAVE_IOS
 #include <ios>
#endif // HAVE_IOS

namespace libglass {

Log *__outlog = NULL;

Log& outlog(void) {
	if (!__outlog) {
 		// std::cerr << "set debug " << std::endl;
 		__outlog = new Log();
// 		__outlog = new Log("out.log");
		__outlog->setLevel(Log::GLASS_DEBUG);
	}
	return *__outlog;
}

} // namespace


using namespace libglass;

Log::Log() : level(GLASS_WARNING), o(&std::cerr) {
}

Log::Log(std::ostream &o) : level(GLASS_WARNING), o(&o) { 
}

Log::Log(const char *filename, bool fifo /* = false */) : level(GLASS_WARNING) { 
	if (fifo == true) {
#if defined(_MSC_VER)
		/* Named pipe (i.e. FIFO) not implemented yet under Win32 */
#else
		if (mkfifo(filename, S_IRWXU | S_IRWXG)) {
			/* error */
		}
#endif // _MSC_VER
	}
	o = new std::ofstream(filename);
	o->setf(std::ios::unitbuf);
}

Log::Log(const char *hostname, int port) {
	// todo
}

Log::~Log() {
	o->flush();
	delete o;
}

std::string Log::getHeader(verboseLevel l) const {
	std::ostringstream		s;
	s << boost::this_thread::get_id();
	return s.str() + "|" + strlevel(l);
}

void Log::header(verboseLevel l) {
	*this << this->getHeader(l);
}

Log & Log::operator<< (const std::string &str) {
	*o << str;
	return *this;
}

Log & Log::operator<< (const char *str) {
	*o << str;
	return *this;
}

bool Log::lock(void) {
	mutex.lock();
	return true;
}

bool Log::unlock(void) {
	mutex.unlock();
	return true;
}


PacketLog::PacketLog() : Log() {
}

PacketLog::PacketLog(std::ostream &o) : Log(o) { 
}

PacketLog::PacketLog(const char *filename, bool fifo /* = false */) : 
		Log(filename, fifo) {
}

PacketLog::PacketLog(const char *hostname, int port) : Log(hostname, port) {
}

void PacketLog::header(verboseLevel l) {
	*this << strlevel(l);
//	*this << PTYPES_NAMESPACE::nowstring("[%c] "); 
/* ALTERNATIVE:
	char buf[128];
	time_t longtime;
	time(&longtime);

	tm t;
	localtime_r(&longtime, &t);
	int r = strftime(buf, sizeof(buf), "[%c] ", &t);

	buf[r] = 0;
	*this << strlevel(l) << buf; 
	*this << strlevel(l); 
	*/
}
