/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "log.h"
#include "serial.h"
#include "objectmanager.h"

using namespace libglass;

SerializableObjectManager::SerializableObjectManager() {
	LOG(outlog(), GLASS_DEBUG, CodeDebug);
}

bool SerializableObjectManager::registerObject(SerializableObject *o) {

	if (!o)
		return false;
	const string name(o->name());
	if (name == "")
		return false;

	LOG(outlog(), GLASS_DEBUG, CodeDebug << ": " << name);

	if (objects.find(name) != objects.end()) {
		LOG(outlog(), GLASS_ERROR, CodeDebug << ": " << "Object exists");
		return false;
	}
	objects[name] = o;

	return true;
}

const SerializableObject *SerializableObjectManager::
			getObject(const string classname) const {
	std::map<const string, const SerializableObject *,
			ConstStringPointerCompare>::const_iterator it;

	it = objects.find(classname);
	if (it == objects.end()) {
		return NULL;
	}

	return it->second;
}

SerializableObject *SerializableObjectManager::
			newInstance(const string classname,
			string *data /* = NULL */) {

	const SerializableObject *o(getObject(classname));
	if (!o) {
		return NULL;
	}

	if (data)
		return o->create(chain(*data));
	else
		return o->create();
}

RegisterObject::RegisterObject(SerializableObject *o) {
	SerializableObjectManager &som =
			 SerializableObjectManager::getHandle();
	
	som.registerObject(o);
}
