/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_OBJECTMANAGER_HH_
#define _GLASS_OBJECTMANAGER_HH_

#include "types.h"
#include "serial.h"
#include <map>

namespace libglass {

/*!
  Manages serializable object registering.
*/
class SerializableObjectManager {
private:
	std::map<const string, const SerializableObject *> objects;

	/*! 
	  Private constructor. This class is a singleton. 
	*/
	SerializableObjectManager();
	
	/*! 
	  Operator= overloading. This class is a singleton.
	*/
	void operator=(SerializableObjectManager &);

	/*! 
	  Avoids cloning. This class is a singleton.
	*/
	SerializableObjectManager(const SerializableObjectManager &);

protected:
	friend class RegisterObject;

	bool registerObject(SerializableObject *o);

public:
	/*! 
	  Returns a handle to this class. This class is a singleton.
	  \return A handle to this class.
	*/
	static SerializableObjectManager& getHandle(void) {
		static SerializableObjectManager *singleton = 
				new SerializableObjectManager();
		return *singleton;
	}

	/*! 
	  \return The internal
	*/
	const SerializableObject *getObject(const string classname) const;

	/*! 
	  \param classname The class name of the new object.
	  \param data The serialized data. If NULL, the default constructor
	    is called.
	  \return a new
	*/
	SerializableObject *newInstance(const string classname,
			string *data = NULL);
};

}

#endif
