/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_PACKET_CLIENT_SERVER_H
#define _GLASS_PACKET_CLIENT_SERVER_H

#include "packet.h"
#include <vector>

namespace libglass {

class ClientServerPacket : public Packet {
public: 
	typedef enum {
		// server->client
		ID = 0, NODE_LIST, SET_MASTER,

		// client->server
		HELLO, QUIT_ALL,
		
		// both
		BYE,

		_CLIENTSERVER_MAX // Always keep this as the last value in the base class

	} ClientServerPacketType;

protected:
	int				type;		/**< packet type */
	nodeId			id;		/**< node id */
	nodeId			master; /**< node id */
	char 			data[9];	/**< packet data */
	std::map<nodeId, nodeNickName>	nodelist;	/**< list of nodes */
	nodeNickName	nickname;

	virtual bool parse(Packet &p);

	/*!
	 * Protected constructor. Used by children classes.
	 * @return
	 */
	ClientServerPacket(Glass *g) throw(Exception);

public:
	/*!
	  Constructor

	  \param p The packet
	  \param autoparse If true, parse the packet. This allows derived classes to call
	  this constructor (with false), otherwise ClientServerPacket::parse() is called
	  since there is not dynamic binding in the base class constructor.
	*/
	ClientServerPacket(Packet &p, bool autoparse = true) throw(Exception);

	/*!
	 * Constructor
	 *
	 * @param g
	 * @param type The ClientServerPacketType, converted to int (the cast is made so
	 * other classes can extend this one).
	 */
	ClientServerPacket(Glass *g, int type) throw(Exception);

	virtual ~ClientServerPacket();

	virtual ClientServerPacketType getType(void) const;

	int getTypeAbstract(void) const {
		return (int)this->type;
	}
	
	/*!
	  Gets the encoded nodeId. Used by ID and NODE_LIST packets.
	  \return The nodeId.
	*/
	nodeId getNodeId(void) const;

	void setNodeId(nodeId id);

	void setNickName(const nodeNickName &nickname) { this->nickname = nickname; }

	const nodeNickName &getNickName(void) const { return this->nickname; }

	nodeId getMasterId(void) const { return this->master; }

	const std::map<nodeId, nodeNickName> &getNodeList(void) const;
	
	void setNodeList(nodeId master, const std::map<nodeId, nodeNickName> &l);
	
	virtual chain encode(unsigned int *len = NULL);

	virtual std::ostream& print(std::ostream &o);
};

} // namespace

#endif // _GLASS_PACKET_CLIENT_SERVER_H
