/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GLASS_PACKET_PEER_H
#define _GLASS_PACKET_PEER_H

#include "packetclientserver.h"
#include <vector>
#include <map>

namespace libglass {

class PeerPacket : public ClientServerPacket {
protected:
	virtual bool parse(Packet &p);

	nodeAddress node; /**< information about this node */

	std::map<nodeId, nodeAddress>	peerlist;	/**< list of peer */

public: 
	typedef enum {
		// server->client
		ID = 0, NODE_LIST,

		// client->server
		HELLO, QUIT_ALL,

		// both
		BYE,

		_CLIENTSERVER_MAX,

		// requests
		PEER_HELLO = 100,

		PEER_START,

		// responses
		PEER_LIST,
		
		PEER_CHANGE_ID,

		// both
		PEER_BYE
	} ClientServerPacketType;

public:
	/*!
	  Constructor
	  \param p The packet
	  \param autostart
	  \return The name of that level.
	*/
	PeerPacket(Packet &p, bool autostart = true) throw(Exception);

	PeerPacket(Glass *g, int t) throw(Exception);

	virtual ~PeerPacket();

	// virtual ClientServerPacketType getType(void) const;

	void setnodeAddress(nodeAddress p) {
		this->node = p;
	}
	const nodeAddress getnodeAddress(void) const {
		return this->node;
	}
	void setPeerList(std::map<nodeId, nodeAddress> &l) {
		this->peerlist = l;
	}
	const std::map<nodeId, nodeAddress>& getPeerList(void) const {
		return this->peerlist;
	}

	virtual chain encode(unsigned int *len = NULL);

	virtual std::ostream& print(std::ostream &o);
};

} // namespace

#endif // _GLASS_PACKET_PEER_H
