/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "protocol.h"
#include "_glass.h"
#include "packet.h"
#include <set>

using namespace libglass;

/*!
  Constructor.
*/
Protocol::Protocol() : owner(NULL) {
}

Protocol::~Protocol() { 
}

void Protocol::setOwner(const Glass *g) { 
	// we can't do much if g == NULL. It's no safer to use the older value.
	this->owner = (Glass *)g;
}

nodeAddress Protocol::IdToAddress(nodeId id) const {
	if (!owner) {
		return nobodyAddress;
	}
	return owner->IdToAddress(id);
}

nodeAddress Protocol::FDToAddress(nodeFD fd) const {
	if (!owner) {
		return nobodyAddress;
	}
	return owner->FDToAddress(fd);
}


nodeId Protocol::AddressToId(const nodeAddress &addr) const {
	if (!owner) {
		return nobodyId;
	}
	return owner->AddressToId(addr);
}

bool Protocol::unregisterNode(nodeId id) {
	if (!owner) {
		return false;
	}
	return owner->unregisterNode(id);
}

int Protocol::listen(unsigned int port) {
	return this->_listen(port);
}

nodeAddress Protocol::accept(void) {
	return this->_accept();
}

nodeAddress Protocol::connect(const char *host, unsigned int port) {
	return this->_connect(host, port);
}

nodeAddress Protocol::connect(const nodeAddress &address) {
	return this->_connect(address);
}


void Protocol::disconnect(const nodeAddress &node) {
	this->_disconnect(node);
	unregisterNode(this->AddressToId(node));
}

bool Protocol::send(Packet &p) {
	const std::set<nodeAddress> &dest(p.getTargetAdresses());
	LOG(outlog(), GLASS_DEBUG, CodeDebug);
	if (dest.empty()) {
		// Sending message to nobody
		LOG(outlog(), GLASS_DEBUG, "destaddr is empty");
		return true;
	}

	return this->_send(p);
}

int Protocol::recv(Packet &p) {
	return this->_recv(p);
}
