/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass barrier plugin tester

Program description:
	General tests (connection)

Dependencies:
	all basic Glass structure.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	31 August 2011: Initial implementation (Bruno)

Return codes:
0	Test ended
******************************************************************************/


#include <iostream>
#include <unistd.h>
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE general
#include <boost/test/unit_test.hpp>
#include "libglass.h"

using namespace libglass;

#define NUM_LOOPS 3
#define NUM_BARRIERS 2

boost::mutex m;


void barrier_thread(int id, int clientnum) {
	GlassClient	*g;
	Barrier	*b;
	unsigned int i, j;

	BOOST_TEST_MESSAGE("Client " << id << " trying to connect to localhost");

	try {
		std::ostringstream oss;
		oss << id;

		g = new GlassClient(new TCP(), "localhost");
		BOOST_REQUIRE(g != NULL);
	}
	catch (Exception &e) {
		BOOST_FAIL("Client constructor failed: " + e.getMessage());
		return;
	}

	BOOST_TEST_MESSAGE("Client connected");

	try {
		boost::mutex::scoped_lock l(m);
		g->setCurrent();
		BOOST_TEST_MESSAGE("Creating barriers");
		b = new Barrier(0, 0, true);
		BOOST_CHECK(b != NULL);
	}
	catch (Exception e) {
		BOOST_FAIL("Barrier " << i << " constructor failed: " + e.getMessage());
	}

	BOOST_TEST_MESSAGE("Created" << id);
	sleep(2);

	b->sync();

	BOOST_TEST_MESSAGE("Synced" << id);

	return;
}


void server_thread() {
	GlassServer		*g;

	BOOST_TEST_MESSAGE("Server starting");

	try {
		g = new GlassServer(new TCP());
		BOOST_CHECK(g != NULL);
	}
	catch (Exception &e) {
		BOOST_FAIL("Server constructor failed: " + e.getMessage());
		return;
	}
	delete g;
	BOOST_TEST_MESSAGE("Server finished");

	return;
}

BOOST_AUTO_TEST_CASE(generalCase) {
	unsigned int clientnum = 3; // amount of clients

	BOOST_TEST_MESSAGE("Total clients: " << clientnum);

	// open server
	boost::thread server(&server_thread);
	boost::thread_group clients;

	unsigned int i;

	for (i = 0; i < clientnum; i++) {
		clients.create_thread(boost::bind(&barrier_thread, i, clientnum));
	}

	BOOST_TEST_MESSAGE("Joining: " << clientnum);

	clients.join_all();

	BOOST_TEST_MESSAGE("Joined");
}
