/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass barrier plugin tester

Program description:
	Barrier tests

Dependencies:
	all basic Glass structure.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>
	Diego Roberto Colombo Dias <diegocolombo@users.sourceforge.net>

History:
	31 August 2011: Initial implementation (Bruno)

Return codes:
0	Test ended
 ******************************************************************************/


#include <iostream>
#include <unistd.h>
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE barrier
#include <boost/test/unit_test.hpp>
#include "libglass.h"

#define NUM_LOOPS 2
#define NUM_BARRIERS 2
#define TOTAL_CLIENTS 2

using namespace libglass;

boost::barrier b(TOTAL_CLIENTS);

boost::mutex m;

void barrier_thread(int id, int clientnum) {
	GlassClient	*g;
	Barrier	*b1;
	Barrier	*b2;

	unsigned int i;

	BOOST_TEST_MESSAGE("Client " << id << " trying to connect to localhost");

	try {
		std::ostringstream oss;
		oss << id;

		g = new GlassClient(new TCP(), "localhost");
		BOOST_REQUIRE(g != NULL);
	}
	catch (Exception &e) {
		BOOST_FAIL("Client constructor failed: " + e.getMessage());
		return;
	}

	BOOST_TEST_MESSAGE("Client connected");

	if (g->isMaster()){
		try {
			boost::mutex::scoped_lock l(m);
			g->setCurrent();
			BOOST_TEST_MESSAGE("Creating barriers");
			b1 = new Barrier(0, 0, true);
			BOOST_CHECK(b1 != NULL);
		}
		catch (Exception e) {
			BOOST_FAIL("Barrier " << i << " constructor failed: " + e.getMessage());
		}


		BOOST_TEST_MESSAGE("Created" << id);
		sleep(2);
		b1->sync();
		BOOST_TEST_MESSAGE("Synced 1" << id);
		sleep(2);

	}else {
		try {
			boost::mutex::scoped_lock l(m);
			g->setCurrent();
			BOOST_TEST_MESSAGE("Creating barriers");
			b2 = new Barrier(0, 0, true);
			BOOST_CHECK(b2 != NULL);
		}
		catch (Exception e) {
			BOOST_FAIL("Barrier " << i << " constructor failed: " + e.getMessage());
		}


		BOOST_TEST_MESSAGE("Created" << id);
		sleep(2);
		b2->sync();
		BOOST_TEST_MESSAGE("Synced 2" << id);
		sleep(2);

	}
	
	b.wait();

	return;
}

void client_thread(int id, int clientnum) {
	GlassClient	*g;

	BOOST_TEST_MESSAGE("Client " << id << " trying to connect to localhost");

		try {
			std::ostringstream oss;
			oss << id;

			g = new GlassClient(new TCP(), "localhost");
			BOOST_REQUIRE(g != NULL);
		}
		catch (Exception &e) {
			BOOST_FAIL("Client constructor failed: " + e.getMessage());
			return;
		}

		BOOST_TEST_MESSAGE("New client connected");
			
}


void server_thread() {
	GlassServer		*g;

	BOOST_TEST_MESSAGE("Server starting");

	try {
		g = new GlassServer(new TCP());
		BOOST_CHECK(g != NULL);
	}
	catch (Exception &e) {
		BOOST_FAIL("Server constructor failed: " + e.getMessage());
		return;
	}
	delete g;
	BOOST_TEST_MESSAGE("Server finished");

	return;
}

BOOST_AUTO_TEST_CASE(barrierCase) {
	unsigned int clientnum = 2; // amount of clients

	BOOST_TEST_MESSAGE("Total clients: " << clientnum);

	// open server
	boost::thread server(&server_thread);
	boost::thread_group clients;

	unsigned int i;

	for (i = 0; i < clientnum; i++) {
		clients.create_thread(boost::bind(&barrier_thread, i, clientnum));
	}
	
	sleep (10);
	
	clients.create_thread(boost::bind(&client_thread, 2, 2));

	BOOST_TEST_MESSAGE("Joining: " << clientnum);

	clients.join_all();

	BOOST_TEST_MESSAGE("Joined");
}
