/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass barrier plugin tester

Program description:
	General tests (connection)

Dependencies:
	all basic Glass structure.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	31 August 2011: Initial implementation (Bruno)

Return codes:
0	Test ended
******************************************************************************/


#include <iostream>
#include <unistd.h>
#include <map>
#include "test.h"
#include "libglass.h"

#include <signal.h>

using namespace libglass;

unsigned int clients = 0; // clients connected so far
bool masterFound = true;

void client_thread(unsigned int id, unsigned int totalclients) {
	GlassClient	*g;

	TEST_MESSAGE("Client trying to connect to localhost");

	try {
		std::ostringstream oss;
		oss << id;

		g = new GlassClient(new PROTOCOLCLASS(), "localhost", "node" + oss.str());
		REQUIRE(g != NULL);
	}
	catch (Exception &e) {
		FAIL("Client constructor failed: " + e.getMessage());
		return;
	}
	TEST_MESSAGE("Client " << boost::this_thread::get_id() << " connected");

	clients++;
	if (g->isMaster()) {
		masterFound = true;
	}
	while (clients < totalclients) {
		sleep(1); // wait for all connections.
	}

	REQUIRE_MESSAGE(masterFound == true, "Master found");

	TEST_MESSAGE("Testing");
	REQUIRE_MESSAGE(totalclients+1 == g->getTotalNodes(),  "Expected " << totalclients+1 << " got " << g->getTotalNodes());

	TEST_MESSAGE("Nicknames and nodelist");
	std::map<nodeId, nodeNickName> nodelist(g->getClientList());
	std::map<nodeId, nodeNickName>::const_iterator it;
	for (int i = 0; i < totalclients; i++) {
		bool found = false;
		std::ostringstream oss;
		oss << id;
		std::string nodename = "node" + oss.str();

		for (it = nodelist.begin(); it != nodelist.end(); it++) {
			std::string s(it->second);
			if (nodename == it->second) {
				found = true;
				break;
			}
		}
		REQUIRE_MESSAGE(found == true,  "Node not found in list " << nodename);
	}

	sleep(3);
	TEST_MESSAGE("Tested");

	delete g;

	TEST_MESSAGE("Client finished");

	return;
}

void server_thread() {
	GlassServer		*g;

	TEST_MESSAGE("Server starting");

	try {
		g = new GlassServer(new PROTOCOLCLASS());
		REQUIRE(g != NULL);
	}
	catch (Exception &e) {
		FAIL("Server constructor failed: " + e.getMessage());
		return;
	}
	delete g;
	TEST_MESSAGE("Server finished");

	return;
}

int main() {
	unsigned int clientnum = 5; // amount of clients

	TEST_MESSAGE("Total clients: " << clientnum);

	// open server
	boost::thread server(&server_thread);
	boost::thread_group clients;
	unsigned int i;

	sleep(2);
	for (i = 0; i < clientnum; i++) {
		clients.create_thread(boost::bind(client_thread, i, clientnum));
	}

	TEST_MESSAGE("Joining: " << clientnum);

	clients.join_all();

	TEST_MESSAGE("Joined");

	TEST_MESSAGE("Test successful");
}
