/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass barrier plugin tester

Program description:
	General tests (isMaster)

Dependencies:
	all basic Glass structure, pthreads.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>
	Marcelo de Paiva Guimaraes<marpaiva@users.sourceforge.net>

History:
	06 March 2005: Initial implementation (Marcelo)

Return codes:
0	Test ended
******************************************************************************/


#include <iostream>
#include <unistd.h>
#include "libglass.h"

using namespace libglass;

//#define MANUAL_DEBUGGING

using std::cout;
using std::endl;


unsigned int clientnum = 3; // amount of clients
unsigned int TOTAL_TESTS=5; // amount of tests

// Struct used to store the tests
struct Test {
	const char			*description; // test executed
	unsigned int		status; //   0=ok  1 = there is a problem
	string                  error; // error returned
};

// tests executed
Test tests[] = {
	{"Server Initialization", 0, },	
	{"isMaster Initialization", 0},
	{"isMaster Test", 0},	
	{"Client Initialization - Not Master", 0},	
	{"Client is not Master", 0}
};



void* client_thread(void *arg) {
	GlassClient		*g;
	unsigned int count=0;
	cout << "Executing test 3: " << tests[3].description << endl;
#ifdef MANUAL_DEBUGGING
	cout << "Client trying to connect to localhost" << endl;
#endif
	try {
		g = new GlassClient(new TCP(), "localhost");
	}
	catch (Exception e) {
		tests[3].status = 1;
		tests[3].error = e.getMessage();
		return NULL;
	}
	cout << "Executing test 4: " << tests[4].description << endl;
	while (count<10){
//#ifdef MANUAL_DEBUGGING
//	        cout << "Testing client" << endl;
//#endif	
//	        cout << "Testing client" << endl;
		if (g->isMaster()){
			tests[4].status = 1;
			tests[4].error = "Simple client is Master";
		}
		sleep(1);
		count++;
	}
	delete g;
#ifdef MANUAL_DEBUGGING
	cout << "Client finished" << endl;
#endif	
	return NULL;

}

void* client_ismaster(void *arg) {
	GlassClient		*g;
	unsigned int count=0;
	cout << "Executing test 1: " << tests[1].description << endl;
#ifdef MANUAL_DEBUGGING
	cout << "isMaster client trying to connect to localhost" << endl;
#endif
	try {
		g = new GlassClient(new TCP(), "localhost");
	}
	catch (Exception e) {
		tests[1].status = 1;
		tests[1].error = e.getMessage();
		return NULL;
	}
	cout << "Executing test 2: " << tests[2].description << endl;
	while (true){
//#ifdef MANUAL_DEBUGGING
//	        cout << "Testing client(master)" << endl;
//#endif	
//	        cout << "Testing client(master)" << endl;
		if (!g->isMaster()){
			tests[2].status = 1;
			tests[2].error = "Error-> Master identification";
		}
		sleep(2);
		count++;
	}
	delete g;
#ifdef MANUAL_DEBUGGING
	cout << "Client (Master) finished" << endl;
#endif	
	return NULL;
}

void* server_thread(void *arg) {
	cout << "Server - Executing test 0: " << tests[0].description << endl;
	GlassServer		*g;
#ifdef MANUAL_DEBUGGING
	cout << "Server starting" << endl;
#endif
	
	try {
		g = new GlassServer(new TCP());
	}
	catch (Exception e) {
		cout <<" Failed" << endl;
		tests[0].status=1;
		tests[0].error = e.getMessage();
		return NULL;
	}
	delete g;
#ifdef MANUAL_DEBUGGING
	cout << "Server finished" << endl;
#endif
	return NULL;
}


int main(int argc, char **argv) {
	unsigned int i;
	pthread_t 		server, masterclient, *clients;

	if (argc > 1) {
		clientnum = atoi(argv[1]);
	}
	cout << "Total of tests : " << TOTAL_TESTS << "\tTotal of clients: " << clientnum << endl;

	pthread_create(&server, NULL, server_thread, NULL);
	
	pthread_create(&masterclient, NULL, client_ismaster, NULL);

	sleep(1); // todo: this should be a trigger, to guarantee that the
		  // masterclient thread is initialized

	
	for (i = 0; i < clientnum; i++) {
		pthread_create(&clients[i], NULL, client_thread, NULL);
		sleep(1);
	}

	for (i = 0; i < clientnum; i++) {
		pthread_join(clients[i], NULL);
	}
	//pthread_kill(masterclient,0);
	cout << endl;

	int flag = 0;
	for (i = 0; i < TOTAL_TESTS; i++) {
		if (tests[i].status == 1) {
			cout << "Error: " << tests[i].description << "\t\tTest number: "<<i<< endl;
			cout << "  Message: " << tests[i].error <<endl;
			flag = 1;
		}else	{
			cout << tests[i].description << ".....OK!" <<endl;
		}
	}
	if (flag == 0)
		cout << endl << "No errors detected: " << TOTAL_TESTS << " tests done" << endl;
	cout <<endl;

	return 0;
}
