/*  Glass - a distributed computing library
 Copyright (C) 2003-2009 Bruno Barberi Gnecco
 Copyright (C) 2009-2010 Corollarium Technologies


 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include "libglass.h"

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER
using namespace libglass;

using std::cout;
using std::cerr;
using std::endl;

void one(int a) {
	cerr << "ONE, nodeId " << a << endl;
}

void two(int a) {
	cerr << "TWO, nodeId " << a << endl;
}

void three(int a) {
	cerr << "THREE, nodeId " << a << endl;
}

void four(int a) {
	cerr << "FOUR, nodeId " << a << endl;
}

void five(int a) {
	cerr << "FIVE, nodeId " << a << endl;
}

int main(int argc, char **argv) {
	GlassClient *g=NULL;
	void (*func)(int);

	try {
		if (argc == 1) {
			new GlassServer(new TCP());
		} else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	} catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}

	Alias<void(*)(int)> *alias = new Alias<void(*)(int)> ("integer", one);
	Barrier *b = new Barrier(0, true);

	alias->addAlias((const char *) "one", one);
	alias->addAlias((const char *) "two", two);
	alias->addAlias((const char *) "three", three);
	alias->addAlias((const char *) "four", four);
	alias->addAlias((const char *) "five", five);
	cout << "I am node " << g->getNodeId() << endl;
	if (g->isMaster()) {
		cout << "I am Master " << endl;
	} else {
		cout << "I am not Master " << endl;
	}
	std::map<nodeId, nodeNickName> clients;
	if (g->isMaster()) {
		clients = g->getClientList();
		cout << "Total of clients " << clients.size() << endl;
	}
	std::map<nodeId, nodeNickName>::iterator i;
	nodeId targetid;
	int qtd = 0;
	while (true) {
		if (g->isMaster()) {
			clients = g->getClientList();
			qtd = 1;
			for (i = clients.begin(); i != clients.end(); i++, qtd = (qtd + 1)%5) {
				targetid = i->first;
				std::cout << " node : " << NODE_FORMAT targetid << std::endl;
				if (qtd == 1) {
					alias->associate(targetid, "one");
				} else if (qtd == 2) {
					alias->associate(targetid, "two");
				} else if (qtd == 3) {
					alias->associate(targetid, "three");
				} else if (qtd == 4) {
					alias->associate(targetid, "four");
				} else {
					alias->associate(targetid, "five");
				}
			}
		}
		SLEEP(1); // seconds
		func = alias->getData();
		func(g->getNodeId());
	}

	delete alias;
	delete b;
	delete g;

	return 0;
}
