/*  Glass - a distributed computing library
    Copyright (C) 2003,2004 Bruno Barberi Gnecco

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass barrier plugin tester

Program description: 
	Tests the barrier plugin. 

Dependencies:
	chain

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History: 
	2003: Initial implementation

Return codes:
0	Test ended successfully.
1	Could not instantiate Glass.
2	Could not create Barrier.
******************************************************************************/


#include <iostream>
#if !defined(_MSC_VER)
#  include <unistd.h> 
#endif // _MSC_VER
#include "libglass.h"


#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

#define WAIT 5

using namespace libglass;

using std::cerr;
using std::endl;

int main(int argc, char **argv) {
	Glass		*g;
	Barrier		*b;
	Barrier         *c;
	int		i = 0;

	/* Let's start with a client/server pair. */
	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}

	/* For first dish, a fresh Barrier. */
	try {
		b = new Barrier(0,false);
		c = new Barrier(1,false);
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(2);
	}

	/* The main course is 60 synchronizations of the barrier. */
	cerr << "Starting" << endl;
//	SLEEP(WAIT); // seconds
	while (true) {
		if (argv[2][0] == '1') {
			cerr << "Locking! 1" << endl;
			b->sync();
		}
		else {
			cerr << "Locking! 2" << endl;
			c->sync();
		}
		sleep(4);
	}

	/* For dessert, delete everything. Note that even if you don't delete
	 * the barrier, deleting the Glass class will clean it anyway, and 
	 * notify the other nodes. It's nicer (and it avoids memory leaking)
	 * to delete the barrier, of course.
	 */
	delete b;
	delete g;
	
	return 0;
}
