/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass event plugin tester

Program description:
	Tests the event plugin with arrays.

Dependencies:
	Glass

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	2010: Initial implementation

Return codes:
0	Test ended successfully.
1	Could not instantiate Glass.
2	Could not create Barrier.
******************************************************************************/

#include <iostream>
#include "libglass.h"


#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

#define EVENTS_TO_SEND	20000
#define ARRAY_SIZE 50000

using namespace libglass;

using std::cout;
using std::endl;

int main(int argc, char **argv) {
	GlassClient	*g;
	char tmpdata[] = "Glass library event test";
	int		tmpdatalen = sizeof(tmpdata);
	int		i = 0;

	// fill our array
/*	char	*originaldata = new char[ARRAY_SIZE];
	for (char *p = originaldata; i < ARRAY_SIZE; *p++, i++) {
		*p = tmpdata[i%tmpdatalen];
	} */

	try {
		if (argc == 1) {
			new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cout << e.getMessage() << endl;
		exit(1);
	}

	Event<char *, rawPack, rawUnpack> *event = new Event<char *, rawPack, rawUnpack>("string");

	if (g->isMaster()) {
		cout << "I'm master" << endl;
		cout << "tmpdatalen" << tmpdatalen << endl;
		for (int i = 0; i < EVENTS_TO_SEND; i++) {
			if (event->enqueueEvent(tmpdata, everybodyId, tmpdatalen) == false) {
				cout << "Error enqueuing string" << endl;
			} else{
				cout << "Enqueuing string" << endl;
			}
			SLEEP(1); // seconds
		}
		/// g->killAll();
	}
	else while(1) {
		cout << "I'm not master" << endl;
		if (event->isEmpty()) {
			cout << "string is empty" << endl;
			SLEEP(1); // seconds
		}
		else {
			char *data = event->getEvent();
			cout << "Received event :" << data << endl;
			delete []data;
		}
	}
	delete g;

	return 0;
}
