/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass event plugin tester

Program description:
	Tests the event plugin.

Dependencies:
	Glass

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	2010: Initial implementation

Return codes:
0	Test ended successfully.
1	Could not instantiate Glass.
2	Could not create Event.
******************************************************************************/

#include <iostream>
#include "libglass.h"


#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

#define EVENTS_TO_SEND	500

using namespace libglass;

using std::cout;
using std::endl;

int main(int argc, char **argv) {
	GlassClient	*g=NULL;
	int		eventnum = 0;

	try {
		if (argc == 1) {
			new GlassServer(new SCTP());
		}
		else {
			g = new GlassClient(new SCTP(), argv[1]);
		}
	}
	catch (Exception e) {
		cout << e.getMessage() << endl;
		exit(1);
	}

	Event<int> *eventint = new Event<int>("integer");
	Event<std::string> *eventstr = new Event<std::string>("string");
	cout << "I am node " << g->getNodeId() << endl;
	if (g->isMaster()) {
		cout << "I am Master" << endl;
	}else{
		cout << "I am not Master " << endl;
	}
	for (int i = 0; i < EVENTS_TO_SEND; i++) {
		if (g->isMaster()) {
			cout << "I am Master " << endl;
			if (eventint->enqueueEvent(eventnum++) == false) {
				cout << "Error enqueuing int" << endl;
			} else {
				cout << "Enqueuing int value " << eventnum << endl;
			}
			if (eventstr->enqueueEvent("Glass library event test") == false) {
				cout << "Error enqueuing string" << endl;
			} else {
				cout << "Enqueuing str value " << "Glass library event test"
						<< endl;
			}
			SLEEP(1); // seconds
		} else {
			cout << "I am not Master " << endl;
			if (eventint->isEmpty()) {
				cout << "Int is empty" << endl;
				SLEEP(1); // seconds
			} else {
				cout << "Received int event :" << eventint->getEvent() << endl;
			}
			if (eventstr->isEmpty()) {
				cout << "string is empty" << endl;
				SLEEP(1); // seconds
			} else {
				cout << "Received str event :" << eventstr->getEvent() << endl;
			}
			SLEEP(1); // seconds
		}
	}
	g->killAll();
	delete g;
	return 0;
}
