/******************************************************************************
LibGlass test program

Program description: 
	This program tests the getTotalNodes() method. By doing this we assert
	that rootBase->getTotalNodes() is working properly, as well as your
	derived Glass classes getTotalNodes() implementation. 

Dependencies:
	Supposes that Protocol and the Glass internals are working correctly.

Authors:
	Bruno Barberi Gnecco

History: 
	Aug 15 2010: implemented


TEST
----
Target test: isMaster()

Test specification:
	This test prints whether this node is master or not. A new master node
	should be created if the last one quits or dies.

******************************************************************************/

#include <iostream>
#include "libglass.h"

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER


using namespace libglass;

using std::cerr;
using std::endl;

#define WAIT_TIME		3

int main(int argc, char **argv) {
	GlassClient		*g;
	bool		n;

	try {
		if (argc == 1) {
			GlassServer *s = new GlassServer(new TCP());
		}
		else if (argc == 2) {
			g = new GlassClient(new TCP(), argv[1]);
		}
		else if (argc == 3) {
			g = new GlassClient(new TCP(atoi(argv[2])), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << "Error while testing " << argv[0] << endl;
		cerr << "Return code: 1" << endl;
		cerr << "Description: " << e.getMessage() << endl;
		exit(1);
	}

	cerr << "Waiting for 3 nodes..." << endl;
	while (true) {
		if (g->isMaster()) {
			cerr << n << "We're MASTER" << endl;
		}
		else {
			cerr << n << "We're CLIENT" << endl;
		}

		SLEEP(WAIT_TIME); // seconds
	}

	SLEEP(WAIT_TIME); // seconds

	delete g;

	return 0;
}
