/******************************************************************************
LibGlass test program

Program description: 
	This program tests the getTotalNodes() method. By doing this we assert
	that rootBase->getTotalNodes() is working properly, as well as your
	derived Glass classes getTotalNodes() implementation. 

Dependencies:
	Supposes that Protocol and the Glass internals are working correctly.

Authors:
	Bruno Barberi Gnecco

History: 
	24 Aug 2003: Initial implementation (BBG)

Return codes:
0	Successful
1	Error instantiating Glass.
X	If the test is unsuccessful, the program remains in infinite loop.

TEST
----
Target test: getTotalNodes() (rootBase/Glass classes).

Test specification:
	This test waits in an infinite loop for 3 clients to connect, and then 
	quits successfully. The number of clients is checked every WAIT_TIME 
	seconds.

******************************************************************************/

#include <iostream>
#include "libglass.h"

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER


using namespace libglass;

using std::cerr;
using std::endl;

#define WAIT_TIME		3

int main(int argc, char **argv) {
	Glass		*g;
	unsigned int	n;

	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << "Error while testing " << argv[0] << endl;
		cerr << "Return code: 1" << endl;
		cerr << "Description: " << e.getMessage() << endl;
		exit(1);
	}

	cerr << "Waiting for 3 nodes..." << endl;
	while ((n = g->getTotalNodes()) < 3) {
		cerr << n << " nodes so far..." << endl;
		SLEEP(WAIT_TIME); // seconds
	}
	cerr << "3 nodes connected, test successful." << endl;

	SLEEP(WAIT_TIME); // seconds

	delete g;

	return 0;
}
