/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass remote plugin tester

Program description: 
	Tests the remote variable plugin. 

Dependencies:
	Glass

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History: 
	23 Jul 2004: Initial implementation

Return codes:
0	Test ended successfully.
1	Could not instantiate Glass.
2	Could not create Remote.
******************************************************************************/


#include <iostream>
#if !defined(_MSC_VER)
#  include <unistd.h> 
#endif // _MSC_VER
#include "libglass.h"

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

using namespace libglass;

using std::cerr;
using std::endl;

int main(int argc, char **argv) {
	GlassClient		*g;
	Remote<int>		*r;
	Barrier			*b;
	int i = 0;

	/* Let's start with a client/server pair. */
	try {
		if (argc == 1) {
			new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}

	try {
		r = new Remote<int>("remoteint", 0);
		b = new Barrier(1);
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(2);
	}

	while (1) {
		if (g->isMaster()) {
			std::map<nodeId, nodeNickName> clients(g->getClientList());
			for (std::map<nodeId, nodeNickName>::const_iterator it(clients.begin()); it != clients.end(); it++) {
				std::cout << "Setting value: " << i << " for " << it->first << std::endl;
				r->setValue(it->first, i++);
			}
		}
		std::cout << "Value (node " << g->getNodeId()  << "): " << r->getData() << std::endl;
		// b->sync();
		SLEEP(1);
	}


	delete r;
	delete g;
	
	return 0;
}
