/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass event plugin tester

Program description:
	Tests the event plugin with arrays.

Dependencies:
	Glass

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	2010: Initial implementation

Return codes:
0	Test ended successfully.
1	Could not instantiate Glass.
2	Could not create Barrier.
******************************************************************************/

#include <iostream>
#include "libglass.h"
#include <signal.h>
#include <stdio.h>

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

#define EVENTS_TO_SEND	20000
#define ARRAY_SIZE 50000

using namespace libglass;

using std::cout;
using std::endl;

int main(int argc, char **argv) {
	GlassClient	*g;
	char	tmpdata[100] = { 0 }, tmpdata2[100] = { 0 }, *p;
	unsigned int tmpdatalen = 100;
	int		i = 0, iteration = 0;
	bool	ret;

	try {
		if (argc == 1) {
			new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cout << e.getMessage() << endl;
		exit(1);
	}

	Shared<char *, rawPack, rawUnpack> *shared = new Shared<char *, rawPack, rawUnpack>("string", NULL);

	while (true) {
		if (g->isMaster()) {
			// fill our array
			cout << "I'm master" << endl;
			if (iteration % 2) {
				tmpdatalen = sprintf(tmpdata, "Glass library test %3d", iteration) + 1;
				p = tmpdata;
			}
			else {
				tmpdatalen = sprintf(tmpdata2, "Second test %3d alternate", iteration) + 1;
				p = tmpdata2;
			}
			cout << "Sharing " << p << endl;
			shared->assign(p, tmpdatalen);
			ret = shared->sendUpdate();
			if (ret == false) {
				cout << "Error enqueuing string" << endl;
			} else {
				cout << "Enqueuing string" << endl;
			}
			SLEEP(1); // seconds
			iteration = (iteration+1)%100;
		}
		else {
			cout << "I'm client" << endl;
			shared->getUpdate();
			cout << "Got update" << endl;
			char *p = shared->getData();
			if (p) std::cerr << "VALUE \"" << p << "\"" << endl;
			SLEEP(1);
		}
	}
	delete g;

	return 0;
}
