/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass packager system tester

Program description: 
	Tests the serialization and unserialization functions.

Dependencies:
	chain

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History: 
	22 Jul 2011

Return codes:

******************************************************************************/


#include <iostream>
#include "types.h"
#include "packager.h"

using namespace std;
using namespace libglass;

#define serializer libglass::textPackBoost
#define unserializer libglass::textUnpackBoost

bool test_string(void) {
	std::string src, dest;
	unsigned int length = 0;

	src = "aaaaaab";

	cout << "Testing test_string." << endl;
	chain c;
	if (serializer<std::string >(c, src) == false) {
		cout << "test_string: error encoding " << endl;
		return false;
	}
	cout << c << endl;
	cout << "Unserialize." << endl;
	if (unserializer<std::string >(c, 0, dest, length) == false) {
		cout << "test_string: error decoding " << endl;
		return false;
	}
	cout << "Compare." << endl;
	if (src != dest) {
		cout << "test_string: src  != dest " << src << "," << dest << endl;
		return false;
	}

	cout << "Passed test_string." << endl;
	return true;
}


bool test_map_string_string(void) {
	std::map<std::string, std::string> src, dest;
	unsigned int length = 0;

	src["a"] = "aaaaaa";
	src["b"] = "bbbbbb";
	src["c"] = "cccccc";

	cout << "Testing test_map_string_string." << endl;
	chain c;
	if (serializer<std::map<std::string, std::string> >(c, src) == false) {
		cout << "test_map_string_string: error encoding " << endl;
		return false;
	}
	cout << c << endl;
	cout << "Unserialize." << endl;
	if (unserializer<std::map<std::string, std::string> >(c, 0, dest, length) == false) {
		cout << "test_map_string_string: error decoding " << endl;
		return false;
	}
	cout << "Compare." << endl;
	if (src != dest) {
		cout << "test_map_string_string: src  != dest " << endl;
		return false;
	}

	cout << "Passed test_map_string_string." << endl;
	return true;
}

int main() {
	unsigned int	 	i;
	bool (*tests[])(void) = {
		test_string,
		test_map_string_string
	};
	int		errors = 0;

	cout << "Executing : " << sizeof(tests)/sizeof(*tests) << " tests." << endl;
	for (i = 0; i < sizeof(tests)/sizeof(*tests); i++) {
		if (tests[i]() == false) {
			errors++;
		}
	}
	cout << errors << " errors detected." << endl;

	return 0;
}
