/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass packet class tester

Program description:
	XXXXXXXXXXXXXXXXx

Dependencies:
	XXXXXXXXXXXXXXXXXXXXXXXXXX

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	05 Oct 2004: Minor fixes
	02 Jul 2004: Updated
	some point of 2003: initial version

Return codes:
0	Finished test.
******************************************************************************/

#include <iostream>
#include "packet.h"

using namespace std;
using namespace libglass;

int main() {
	Packet		p(NULL);
	chain		c;

	p.setPluginId("PLUGIN");
	p.setData("abcdefgh", 9);
	p.setSenderNode(0);

	cout << "Testing setData, getDataLength." << endl;
	cout << "Size." << endl;
	cout << "Expected: 9" << endl;
	cout << "Received: " << p.getDataLength() << endl << endl;

	cout << "Testing setData, setPluginId, setSenderNode, encode." << endl;
	cout << "Expected: 0|21|0|0|0|30|0|0|0|0|0|0|0|0|80|76|85|71|73|78|0|97|98|99|100|101|102|103|104|0|"
			<< endl;
	c = p.encode();
	cout << "Received: ";
	for (unsigned int i = 0; i < c.length(); i++) {
		cout << (unsigned int)c[i] << "|";
	}
	cout << endl << endl;
	// TODO: check if expected and result match

	cout << "Testing appendData." << endl;

	p.appendData("123456789", 10);
	cout << "Size." << endl;
	cout << "Expected: " << endl;
	cout << "Received: " << p.getDataLength() << endl;

	cout << "Expected: 0|21|0|0|0|40|0|0|0|0|0|0|0|0|80|76|85|71|73|78|0|97|98|99|100|101|102|103|104|0|49|50|51|52|53|54|55|56|57|0|"
			<< endl;
	c = p.encode();
	cout << "Received: ";
	for (unsigned int i = 0; i < c.length(); i++) {
		cout << (unsigned int)c[i] << "|";
	}
	cout << endl << endl;

	cout << "Testing decode." << endl;

	Packet p2(NULL);
	p2.decode(c);

	cout << "PluginId." << endl;
	cout << "Expected: PLUGIN" << endl;
	cout << "Received: " << p2.getPluginId() << endl;
	cout << "SenderNode." << endl;
	cout << "Expected: 0" << endl;
	cout << "Received: " << p2.getSenderNode() << endl;
	cout << "Data." << endl;
	cout << "Expected: 97|98|99|100|101|102|103|104|0|49|50|51|52|53|54|55|56|57|0|" << endl;
	cout << "Received: ";
//	unsigned char *b = p2.getData(); BUG!!!!
	chain r;
	p2.getData(r);
	for (unsigned int i = 0; i < p2.getDataLength(); i++) {
		cout << (int)r[i] << "|";
	}
	cout << endl << endl;

	cout << endl << "Testing alternate decode" << endl;

	unsigned char header[] =  { 0, 21, 0, 0, 0, 31, 0, 0, 0, 0, 0, 0, 0, 0, 80, 76, 85, 71, 73, 78, 0 };
	unsigned char data[] = "123456789";
	unsigned char *dyndata = new unsigned char[sizeof(data)];
	memcpy(dyndata, data, sizeof(data));

	p.decode(header, 21, dyndata, 10);

	cout << "PluginId." << endl;
	cout << "Expected: PLUGIN" << endl;
	cout << "Received: " << p2.getPluginId() << endl;
	cout << "SenderNode." << endl;
	cout << "Expected: 0" << endl;
	cout << "Received: " << p2.getSenderNode() << endl;
	cout << "Data." << endl;
	cout << "Expected: 49|50|51|52|53|54|55|56|57|0|" << endl;
	cout << "Received: ";
	p.getData(r);
	for (unsigned int i = 0; i < p.getDataLength(); i++) {
		cout << (int)r[i] << "|";
	}
	cout << endl << endl;

	return 0;
}
