/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/******************************************************************************
LibGlass chain tester

Program description:
	Tests the chain class

Dependencies:
	None.

Authors:
	Marcelo de Paiva Guimaraes <marcelopaivag@yahoo.com.br>
	Bruno Barberi Gnecco

History:
	06 Oct 2004: New tests, fixed old tests
	04 Jul 2004: Minor fixes
	Jan 2004: Initial version

Return codes:
0	On completion. The number of successful and failed tests is printed.

******************************************************************************/

#include <iostream>
#include <map>
#include "chain.h"

using std::cerr;
using std::endl;
using namespace libglass;

int number = 0;

struct S {
	const char			*description;
	const char			*expected;
	unsigned int		expected_size;
};

S tests[] = {
	{"size() and chain::chain(const char *p, unsigned int s/* = 0 */)",	// test 0
	 "", 6},
	{"clear and chain::chain(const char *p, unsigned int s/* = 0 */)",	// test 1
	 "", 0},
	{"append(const char *p, unsigned int s , bool free,char *allocatedbegin )",	// test 2
	 "AAA", 4},
	{"size() and unsigned int chain::length(void)",	// test 3
	 "AAA\0BBBB\0CCCCC", 15},
	{"append(const char *p, unsigned int s , bool free,char *allocatedbegin )",	// test 4
	 "AAA\0BBBB\0CCCCC", 15},
	{"char &chain::operator[](unsigned int position)",	// test 5
	 "AAA\0BBBB\0CCCCC", 15},
	{"bool chain::operator==(const chain &s)",	// test 6
	 "", 0},
	{"bool chain::operator!=(const chain &s)",	// test 7
	 "", 0},
	{"bool chain::operator!=(const chain &s)",	// test 8
	 "", 0},
	{"const chain &chain::operator+=(char *p)",	// test 9
	 "AAA\0xxx", 8},
	{"const chain &chain::operator+=(const chain &s)",	// test 10
	 "AAA\0xxx\0FF", 11},
	{"void chain::appendcopy(chain &s)",	// test 11
	 "hh\0eee\0eee\0gg", 14},
	{"unsigned int chain::find(char c,unsigned int initial /* = 0 */) - !find",	//test 12
	 "", 0},
	{"unsigned int chain::find(char c,unsigned int initial /* = 0 */) - find",	//test 13
	 "", 0},
	{"unsigned int chain::find(char c,unsigned int initial /* = 0 */) - find('A',3)",	//test 14
	 "", 0},
	{"unsigned int chain::find(char c,unsigned int initial /* = 0 */) - find('B',2)",	//test 15
	 "", 0},
	{"unsigned int chain::find(char c,unsigned int initial /* = 0 */) - find('B',2)and append",	//test 16
	 "", 0},
#ifdef DEPRECATED
	{"chain *chain::concatenate(int begin, int end)",
	 "abcdef", 7},
#endif
	{"void chain::append(const chain &s)",	//test 17
	 "Aee\0Aee", 8},
	{"void chain::append(const chain &s)",	//test 18
	 "Aee", 4},
	{"void chain::appendcopy(const char *p, unsigned int s /* = 0 */)",	//test 19
	 "Aee\0eee", 8},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr",	//test 20
	 "AAAA\0BBBB\0CCCC\0DDDD\0EEEE\0FFFF", 30},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 2,6",	//test 21
	 "AA\0BB", 5},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 6,12",	//test 22
	 "BBB\0CCC", 7},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 2,12",	//test 23
	 "AA\0BBBB\0CCC", 11},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 5,50",	//test 24
	 "BBBB\0CCCC\0DDDD\0EEEE\0FFFF\0", 25},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 10,5",	//test 25
	 "", 0},
	{"chain chain::substr(unsigned int begin, unsigned int end) - substr 4,9",	//test 26
	 "\0BBBB\0", 6},
	{"bool chain::substrcopy(char *dest, unsigned int begin, unsigned int end) - substrcopy 40,50",	//test 27
	 "CCCC\0DDDD\0E", 11},
	{"duplicate()",		//test 28
	 "TTTXXX", 7},
	{"chain::chain(std::string &s)",	//test 29
	 "stringtest", 10},
	{"chain::chain(std::string &s)",	//test 30a
	 "stringtest", 11},
	{"void chain::append(std::string &s)",	//test 30
	 "testappendstring", 16},
	{"const chain &chain::operator+=(std::string &s)",	//test 31
	 "AA\0DD", 5},
	{"chain::appendcopy(std::string &s)",	//test 32
	 "XX\0testappendcopy", 18},
	{"chain::erase(chain_iterator it)",	//test 33
	 "", 0},
	{"chain::toArray()",	//test 34
	 "AABBCC", 7},
	{"chain::toString()",	//test 35
	 "ABCD\0EF\0GHI\0JKLMN\0N", 20},
	{"chain::toString(x, x)", //test 36
	 "D\0EF\0GHI\0J", 10},
	{"chain::toString()",	//test 37
	 "ZXCVB\0LKJHGFDSAQ\0POIUJHGFVDSAXZA", 33}
};

static int testcmp(const char *received, int sizer, const char *expected, int sizee) {
	if (sizer != sizee)
		return 1;
	return memcmp(received, expected, sizee);
}

static int compare(chain &c, const char * expected, int size) {
	if ((int)c.size() != size) {
		return 1;
	}
	for (int i = 0; i < size; i++) {
		if (c[i] != expected[i]) {
			return 1;
		}
	}
	return 0;
}

static int compare(string &s, const char *expected, int size) {
	if ((int)s.size() != size) {
		return 1;
	}
	for (int i = 0; i < size; i++) {
		if (s[i] != expected[i]) {
			return 1;
		}
	}
	return 0;
}

#define MANUAL_DEBUGGING
#ifdef MANUAL_DEBUGGING
static void X(chain t) {
	cerr << "Test " << tests[number].description << "    Size found: " << t.size() << "    Size expected: " << tests[number].expected_size << endl;
	cerr << "Char Found    -  Char expected"<< endl;
	for (int i = 0; i < (int)t.size(); i++) { \
		cerr <<"       "<< t[i] << "     -     " << tests[number].expected[i] << endl; \
	}
	cerr << "------------" << endl;
	return;
}
#else
static void X(chain t) {
	return;
}
#endif

#define TOTAL_TESTS	sizeof(tests)/sizeof(S)

int main()
{
	int error[TOTAL_TESTS];
	unsigned int i;

	cerr << "Total of tests " << TOTAL_TESTS << endl;

	chain s("aaaaa");
	chain t("");
	chain m("");
	for (i = 0; i < TOTAL_TESTS; i++)
		error[i] = 0;

	/* test 0 - size() and clear()*/
	if (s.size()!=tests[number].expected_size)
		error[number] = 1;

	/* test 1 - clear()*/
	number++;
	s.clear();
	if (s.size() != tests[number].expected_size)
		error[number] = 1;

	/* test 2  - append(const char *p, unsigned int s , bool free,	char *allocatedbegin )*/
	number++;
	s.clear();
	s.append("AAA");
	if (compare(s, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}

	/*test 3-size() */
	number++;
	s.clear();
	s.append("AAA");
	s.append("BBBB");
	s.append("CCCCC");
	if (s.size() != tests[number].expected_size){
		error[number] = 1;
		X(s);
	}

	/*test 4-append() */
	number++;
	s.clear();
	s.append("AAA");
	s.append("BBBB");
	s.append("CCCCC");
	if (compare(s, tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}
	/*test 5-operator [] */
	number++;
	s.clear();
	s.append("AAA");
	s.append("BBBB");
	s.append("CCCCC");
	for (i = 0; i < tests[number].expected_size; i++) {
		//if (s[i] != teststr[i]) {
		if (s[i] != tests[number].expected[i]) {
			error[number] = 1;
			X(s);
			break;
		}
	}

	/* test 6  operator== */
	number++;
	s.clear();
	t.clear();
	if (!(s == t)){
		error[number] = 1;
		X(s);
	}

	/* test 7  operator=!  */
	number++;
	s.clear();
	t.clear();
	if (s != t){
		error[number] = 1;
		X(s);
	}

	/* test 8 operator! =  */
	number++;
	s.clear();
	t.clear();
	s.append("AAA");
	t.append("FF");
	if (!(s != t)){
		error[number] = 1;
		X(s);
	}

	/* test 9 +='xxx' */
	number++;
	s.clear();
	t.clear();
	s.append("AAA");
	t.append("FF");
	s += "xxx";
	if (compare(s, tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}
	/* test 10  chain += chain */
	number++;
	s.clear();
	t.clear();
	s.append("AAA");
	t.append("FF");
	s += "xxx";
	s += t;
	if (compare(s,tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}
	/* test 11  appendcopy(chain) */
	number++;
	s.clear();
	t.clear();
	s.append("eee");
	t.append("hh");
	t.appendcopy(s);
	s.append("gg");
	t.append(s);
	if (compare(t, tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}
	t.clear();

	/* test 12  !find */
	number++;
	s.clear();
	if (!(s.find('A') > s.size())){
		error[number] = 1;
		X(s);
	}

	/* test 13  find */
	number++;
	s.clear();
	s.append("eAeB");
	if (s.find('A') != 1){
		error[number] = 1;
		X(s);
	}

	/* test 14  find */
	number++;
	s.clear();
	s.append("eAeB");
	if (!(s.find('A', 3) > s.size())){
		error[number] = 1;
		X(s);
	}

	/* test 15  find('B',2) */
	number++;
	s.clear();
	s.append("eAeB");
	if (s.find('B', 2) != 3){
		error[number] = 1;
		X(s);
	}

	/* test 16  find('B',2) and append */
	number++;
	s.clear();
	s.append("eAeB");
	s.append("XY");
	if (s.find('Y', 2) != 6){
		error[number] = 1;
		X(s);
	}

#ifdef DEPRECATED
	/* test 17 concatenate() */
	number++;
	s.clear();
	s.append("abcdef");
	chain *f = s.concatenate();
	if (compare(*f, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(*f);
	}
#endif
	/* test 18  void chain::append(const chain &s) */
	number++;
	s.clear();
	t.clear();
	char p[] = "eee";
	s.append(p);
	t.append(s);
	s.append(p);
	p[0] = 'A';
	if (compare(s, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}

	/* test 19  append */
	number++;
	s.clear();
	t.clear();
	char z[] = "eee";
	s.append(z);
	t.append(s);
	s.append(z);
	z[0] = 'A';
	if (compare(t, tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 20  appendcopy (char *p) */
	number++;
	s.clear();
	t.clear();
	char r[]  = "eee";
	s.append(r);
	s.appendcopy(r);
	r[0] = 'A';
	if (compare(s, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(s);
	}
	/* test 21 substr */
	number++;
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(0, chain::npos);
	if (compare(t, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 22 substr 2,6 */
	number++;
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(2, 6);
	if (compare(t, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 23 substr 6,12 */
	number++;
	t.clear();
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(6, 12);
	if (compare(t, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 24 substr 2,12 */
	number++;
	t.clear();
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(2, 12);
	if (compare(t, tests[number].expected,tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 25 substr 5,50 */
	number++;
	t.clear();
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(5, 50);
	if (compare(t, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 26 substr 10,5 */
	number++;
	t.clear();
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	try {
		t = s.substr(10, 5);
		error[number] = 1;
		X(t);
	}
	catch (Exception e) {
		; // expected
	}

	/* test 27 substr 4,9 */
	number++;
	t.clear();
	s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");
	t = s.substr(4, 9);
	if (compare(t, tests[number].expected, tests[number].expected_size)){
		error[number] = 1;
		X(t);
	}

	/* test 28 substrcopy 40,50 */
	number++;
	t.clear();
        s.clear();
	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");

	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");

	s.append("AAAA");
	s.append("BBBB");
	s.append("CCCC");
	s.append("DDDD");
	s.append("EEEE");
	s.append("FFFF");

	s.append("AAAA");
	s.append("BBBB");

	char test_copy[11];
	s.substrcopy(test_copy,40,50);
	if (testcmp(test_copy, tests[number].expected_size, tests[number].expected,tests[number].expected_size) != 0) {
		error[number] = 1;
		X(s);
	}

	/* test 29 duplicate  */
	number++;
	chain q("");
	s.clear();
	s.append("TTTXXX");
	q =s.duplicate();
	if (compare(q,  tests[number].expected,  tests[number].expected_size)){
		error[number] = 1;
		X(q);
	}

	/* test 30 chain::chain(std::string &s)*/
	{
		number++;
		string xx("stringtest");
		chain w(xx);
		if (compare(w,  tests[number].expected, tests[number].expected_size)){
			error[number] = 1;
			X(w);
		}
	}

	/* test 30a chain::chain(std::string &s)*/
	{
		number++;
		string xx("stringtest", strlen("stringtest")+1);
		chain w(xx);
		if (compare(w,  tests[number].expected, tests[number].expected_size)){
			error[number] = 1;
			X(w);
		}
	}

	/* test 31 void chain::append(std::string &s)*/
	{
		number++;
		string xx="testappendstring";
		s.clear();
		s.append(xx);
		if (compare(s,  tests[number].expected, tests[number].expected_size)){
			error[number] = 1;
			X(s);
		}
	}

	/* test 32 chain &chain::operator+=(std::string &s)*/
	{
		number++;
		string xx="DD";
		s.clear();
		s.append("AA");
		s+=xx;
		if (compare(s,  tests[number].expected,  tests[number].expected_size)){
			error[number] = 1;
			X(s);
		}
	}

	/* test 33 chain::appendcopy(std::string &s)*/
	{
		number++;
		s.clear();
		s.append("XX");
		string xx="testappendcopy";
		s.appendcopy(xx);
		if (compare(s,  tests[number].expected,  tests[number].expected_size)){
			error[number] = 1;
			X(s);
		}
	}

	/* test 34 chain::erase(chain_iterator it) */
	number++;
	s.clear();
	s.append("XX");
	chain_iterator it;
	s.erase(it);
	//TODO
	//if (s.size() != 0){
	//	error[number] = 1;
	//     X(s);
	//}

	/* test 35 chain::toArray() */
	char *from;
	number++;
	s.clear();
	s.append("AABBCC");
	from = s.toArray();
	if (testcmp(from, s.size(), tests[number].expected, tests[number].expected_size) !=0){
		error[number] = 1;
		X(s);
	}
	delete []from;

	/* test 36 chain::toString() */
	{
		number++;
		s.clear();
		s.append("ABCD");
		s.append("EF");
		s.append("GHI");
		s.append("JKLMN");
		s.append("N");
		string st = s.toString();
		if (compare(st, tests[number].expected, tests[number].expected_size)) {
			error[number] = 1;
			cerr << "Test " << tests[number].description << ". Size found: " << st.size() << " Size expected: " << tests[number].expected_size << endl;
//			X(st);
		}
	}

	/* test */
	{
		number++;
		string st = s.toString(3, 12);
		if (compare(st, tests[number].expected, tests[number].expected_size)) {
			error[number] = 1;
			cerr << "Test " << tests[number].description << ". Size found: " << st.size() << " Size expected: " << tests[number].expected_size << endl;
//			X(st);
		}
	}

	/* test 37 chain::toString() */
	{
		number++;
		s.clear();
		s.append("ZXCVB");
		s.append("LKJHGFDSAQ");
		s.append("POIUJHGFVDSAXZA");

		string st = s.toString();
		if (compare(st, tests[number].expected, tests[number].expected_size)) {
			error[number] = 1;
			cerr << "Test " << tests[number].description << ". Size found: " << st.size() << " Size expected: " << tests[number].expected_size << endl;
//			X(st);
		}

		chain cs(st);
		if (st.size() != cs.size()) {
			cerr << "Error string->chain" << endl;
		}
	}
	
	/* test: garbage collection */
	{
		int length = 2000;
		char *data = new char[length];
		s.clear();
		s.append(data, length, true);
	}
	
	int flag = 0;
	for (i = 0; i < TOTAL_TESTS; i++) {
		if (error[i] == 1) {
			cerr << "Error in test number: " << i << " - " << tests[i].description << endl;
			flag = 1;
		}
	}
	if (flag == 0)
		cerr << "No errors detected: " << TOTAL_TESTS << " tests done" << endl;

	if (number != TOTAL_TESTS - 1)
		cerr << "Variable `number` is with wrong value - check this program" <<
			endl;
	return 0;
}
