/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /******************************************************************************
LibGlass test program

Program description:
	Tests the Interprocess memory mechanism.

Dependencies:
	Independent of any other part of libglass.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History:
	12 Aug 2011: Initial implementation (BBG)

Return codes:
0	Successful.

******************************************************************************/

#include <iostream>
#include "interprocess.h"

using namespace libglass;

using std::cout;
using std::endl;

class X {
public:
	int i;
	float f;

	X(int i, float f) : i(i), f(f) {
		cout << "X constructor" << endl;
	}
	~X() {
		cout << "X destructor" << endl;
	}
};

int main() {
	Interprocess i;
	std::string shname("shname");

	X *data = new(i, shname) X(3, 5.25);

	{
		Lock l(i.scoped_lock());
		data->i = 2;
	}

	data->~X(); // placement new requires explicit destructor call
	i.deallocate(data);

	return 0;
}
