/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 /******************************************************************************
LibGlass test program

Program description: 
	This program tests the compiler to ensure that plugins are properly
	loaded before Glass is used.

Dependencies:
	Supposes that the plugin system is working, as it only prints the 
	debugging data. As so, LOG() must be set to DEBUG.

Authors:
	Bruno Barberi Gnecco <brunobg@users.sourceforge.net>

History: 
	12 Sep 2003: Initial implementation (BBG)

Return codes:
0	Successful. The executable can't detect errors.

******************************************************************************/

#include <iostream>
#include "libglass.h"

using namespace libglass;

using std::cout;
using std::endl;

class ProtocolDummy : public Protocol {
protected:
	int _listen(unsigned int port) { return 0; }
	nodeAddress _accept(void) { return nobodyAddress; }
	nodeAddress _connect(const char *host, unsigned int port) { return nobodyAddress; }
	nodeAddress _connect(const nodeAddress &host) { return nobodyAddress; }
	void _disconnect(const nodeAddress &node) { }
	bool _send(Packet &p) { return false; }
	int _recv(Packet &p) { return 0; } 
public:
	ProtocolDummy() { }
	~ProtocolDummy() { }
};

class GlassDummy : public Glass {
private:

protected:
	virtual bool rootPacket(Packet &p) {
		return true;
	}

	// very simple, only one node at a time.
	virtual nodeId registerNode(nodeAddress &fd, nodeId id, nodeRelation r, bool force) {
		return baseServerId;
	}
public:
	GlassDummy(Protocol *p) throw(Exception) : Glass(p) {
		;
	}

	~GlassDummy() {
		cout << endl << "\tDestructor." << endl << endl;
		Glass::clean();
	}

	unsigned int getTotalNodes(void) {
		return 0;
	}
	
	void printPlugins(void) {
		pm->printPlugins();
	}
};

int main(int argc, char **argv) {
	GlassDummy	*g;

	cout << endl << "\tInitializing." << endl << endl;
	try {
		g = new GlassDummy(new ProtocolDummy());
	}
	catch (Exception e) {
		return 0;
	}

	g->printPlugins();
	
	cout << endl << "\tClosing." << endl << endl;
	delete g;

	cout << endl << "\tQuit." << endl << endl;

	return 0;
}
