/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <cstdio>
#include "libglass.h"


using namespace libglass;
using std::cout;
using std::endl;

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER

#define EVENTS_TO_SEND	10

// returns child pid
int server_peer(int port, int remote) {
	int pid;
	pid = fork() ;
	if (pid == -1) {
		cout << " Error: forking peer"<<  endl;
		return 0;
	}
	else if (pid != 0) { // server
		return pid;
	}

	char logname[100];
	sprintf(logname, "server_%d.log", port);
	mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
	int fd = open(logname, O_WRONLY | O_CREAT | O_TRUNC, mode);
	dup2(fd, 1); // redirect stdout
	dup2(fd, 2); // redirect stderr

	cout << "Starting Server " << port << endl;

	GlassServerPeer *g;
	try {
		if (remote)
			g = new GlassServerPeer(new TCP(port), "localhost", remote);
		else
			g = new GlassServerPeer(new TCP(port));
	}
	catch (Exception e) {
		cout << "Failed server " << e.getMessage() << endl;
		exit(0);
	}
	delete g;
	exit(0);
	return 0;
}

/**
 * Opens a client instance
 *
 * @param port Port to connect to
 * @param id Local id, used for the log filename only (nothing to do with glass)
 * @param isglobalmaster If true, then send global events.
 * @return
 */
int client_event(int port, int id, bool isglobalmaster = false) {
	int pid;
	pid = fork() ;
	if (pid == -1) {
		cout << " Error: forking client" <<  endl;
		return 0;
	}
	else if (pid != 0) { // server
		return pid;
	}

	cout << "Client forked " << port << endl;

	char logname[100];
	sprintf(logname, "client_%d_%d.log", port, id);
	mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
	int fd = open(logname, O_WRONLY | O_CREAT | O_TRUNC, mode);
	dup2(fd, 1); // redirect stdout
	dup2(fd, 2); // redirect stderr

	cout << "Client to server " << port << endl;
	GlassClient *g;
	try {
		cout << "Starting Client...  " << endl;
		g = new GlassClient(new TCP(port), "localhost");
		cout << "Client started.................. " << endl;
	}
	catch (Exception e) {
		cout << "Failed client " << port << " :" << e.getMessage() << endl;
		return 0;
	}

	while (true) {
		if (g->isMaster()) {
			cout << "I'm master" << endl;
		}
		else {
			cout << "I'm client" << endl;
		}
		cout << "My id " << g->getNodeId() << endl;

		std::map<nodeId, nodeNickName> clients(g->getClientList());
		std::map<nodeId, nodeNickName>::iterator i;

		for (i = clients.begin(); i != clients.end(); i++) {
			cout << "Other clients " << i->first << endl;
		}

		SLEEP(1);
	}
	delete g;
	exit(0);
	return 0;
}

int main(int argc, char **argv) {
	server_peer(7001, 0);
	SLEEP(1);
	server_peer(8001, 7001);
	//SLEEP(1);
	client_event(7001, 1, true);
	SLEEP(1);
	client_event(8001, 2);
	SLEEP(1);
	client_event(7001, 3);
	while (true) {
		SLEEP(1);
	}
	return 0;
}
