/*  Glass - a distributed computing library
 Copyright (C) 2003-2009 Bruno Barberi Gnecco
 Copyright (C) 2009-2010 Corollarium Technologies


 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <unistd.h>
#include "libglass.h"

using namespace libglass;
using std::cout;
using std::endl;

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER
#define EVENTS_TO_SEND	50

int main(int argc, char **argv) {
	GlassClient *g;
	int eventnum = 0;
	try {
		if (argc == 1) {
			cout << "peerevent [host] " << endl;
			return 0;
		} else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	} catch (Exception e) {
		cout << e.getMessage() << endl;
		return 0;
	}
	Event<int> *eventint = new Event<int> ("integer");
	if (g->isMaster()) {
		cout << "I am Master" << endl;
		cout << "Master started with success " << endl;
		for (int i = 0; i < EVENTS_TO_SEND; i++) {
			if (eventint->enqueueEvent(eventnum++) == false) {
				cout << "Error enqueuing int" << endl;
			} else {
				cout << "Enqueuing int value " << eventnum << endl;
			}
			SLEEP(1); // seconds
		}
		g->killAll();
	} else {
		cout << "I am Client " << endl;
		cout << "Client started with success " << endl;
		cout << "Client node " << g->getNodeId() << endl;
		while (1) {
			if (eventint->isEmpty()) {
				cout << "Int is empty" << endl;
				SLEEP(1); // seconds
			} else {
				cout << "Received event " << eventint->getEvent() << endl;
			}
		}
		delete g;
		return 0;
	}
}
