/*  Glass - a distributed computing library
 Copyright (C) 2003-2009 Bruno Barberi Gnecco
 Copyright (C) 2009-2010 Corollarium Technologies


 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <unistd.h>
#include "libglass.h"

using namespace libglass;
using std::cout;
using std::endl;

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER
#define DATA_TO_SHARE	50

int main(int argc, char **argv) {
	GlassClient *g;
	int i = 0;
	try {
		if (argc == 1) {
			cout << "peershared [host] " << endl;
			return 0;
		} else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	} catch (Exception e) {
		cout << e.getMessage() << endl;
		return 0;
	}
	Shared<int, rawPack, rawUnpack> *sharedint = new Shared<int, rawPack,
			rawUnpack> ("sharedint");
	cout << "Value before update: " << sharedint->getData() << endl;
	//	SLEEP(5); // seconds
	cout << "I am node " << g->getNodeId() << endl;

	while (i <= DATA_TO_SHARE) {
		if (g->isMaster()) {
			cout << "I am Master " << endl;
			*sharedint = i++;
			cout << "Updating sharedint..." << sharedint->getData() << endl;
			sharedint->sendUpdate();
			*sharedint = 0;
		} else {
			cout << "I am not Master " << endl;
		}
		cout << "getting update" << endl;
		sharedint->getUpdate();
		//	_Shared::syncAll();
		cout << "new value: " << sharedint->getData() << endl;
		SLEEP(20); // seconds
	}
	g->killAll();
	delete sharedint;
	return 0;
}
