/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/******************************************************************************
LibGlass thread safety tester.

Program description: 
	Two threads are started. Each one has a barrier
	The thread_barrier1 is slower than thread_barrier2

Dependencies:
	Glass, Barriers.	

Authors:
	Marcelo de Paiva Guimaraes <marcelopaivag@yahoo.com.br>

History:
	Jan 2004 Initial version

Return codes:
0	Finished test. See printed data to see if it was successful.

******************************************************************************/

#include <iostream>
#include "thread.h"

using namespace libglass;

using std::cerr;
using std::endl;

class thread1 : public libglass::GlassThread {
public:
	thread1();
	void execute(void);
	void cleanup(void);
};


// Constructor
thread1::thread1() : libglass::GlassThread() {
}

void thread1::execute() {
	std::cout << "thread 1" << std::endl;
}

void thread1::cleanup() {
}



class thread2: public libglass::GlassThread {
public:
	// Constructor
	thread2();
	void execute(void);
	void cleanup(void);
};


// Constructor
thread2::thread2() : libglass::GlassThread() {
}

void thread2::execute() {
	std::cout << "thread 2" << std::endl;
}

void thread2::cleanup() {
}




int main(int argc, char **argv) {
	thread1 	*th_1 = new thread1;
	thread2		*th_2 = new thread2;

  // start first thread
	th_1->start();
	// start second thread
	th_2->start();

	// Wait for both threads to terminate
	th_1->join();
	th_2->join();

	return 0;
}

