/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/******************************************************************************
LibGlass thread safety tester.

Program description: 
	Two threads are started. Each one has a barrier
	The thread_barrier1 is slower than thread_barrier2

Dependencies:
	Glass, Barriers.	

Authors:
	Marcelo de Paiva Guimaraes <marcelopaivag@yahoo.com.br>

History:
	Jan 2004 Initial version

Return codes:
0	Finished test. See printed data to see if it was successful.

******************************************************************************/


#include <iostream>
#include "libglass.h"

#if defined(_MSC_VER)
#  define SLEEP(a) Sleep(a * 1000)
#else
#  define SLEEP(a) sleep(a)
#endif // _MSC_VER


#define WAIT 15

using namespace libglass;

using std::cerr;
using std::endl;

#define BARRIER_COUNT	3

Glass *g;

class thread1: public GlassThread {
public:
	// Constructor
	thread1();
	void execute(void);
	void cleanup(void);
};


// Constructor
thread1::thread1() : GlassThread(false) {
}

void thread1::execute() {
	int cont=0;	
	Barrier *b= new Barrier(1, true);
	// Pause whilst clients connect and create their barriers
	SLEEP(WAIT); // seconds
	while (cont < BARRIER_COUNT) {
		b->sync();
		cerr <<"Thread 1 -> synch number = " << cont <<endl;
		cont++;
		SLEEP(2); // seconds
	}
	delete b;
}

void thread1::cleanup() {
}



class thread2: public GlassThread {
public:
	// Constructor
	thread2();
	void execute(void);
	void cleanup(void);
};


// Constructor
thread2::thread2() : GlassThread(false) {
}

void thread2::execute() {
	int cont=0;	
	Barrier *b= new Barrier(2, true);
	// Pause whilst clients connect and create their barriers
	SLEEP(WAIT); // seconds
	while (cont < BARRIER_COUNT)		
		{
		b->sync();
		cerr <<"Thread 2 -> sync number = "<< cont <<endl;
		cont++;
		SLEEP(1);	// seconds
	}
	delete b;
}

void thread2::cleanup() {
}




int main(int argc, char **argv) {
	Glass 		*g;
	thread1 	*th_1 = new thread1;
	thread2		*th_2 = new thread2;

	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch(Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}


  // start first thread
	th_1->start();
	// start second thread
	th_2->start();

	// Wait for both threads to terminate
	th_1->join();
	th_2->join();

	delete g;

	return 0;
}

