/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/******************************************************************************
LibGlass thread safety tester.

Program description: 
	Tests thread safety and barriers.

Dependencies:
	Glass, Barriers.

Authors:
	Marcelo de Paiva Guimaraes <marcelopaivag@yahoo.com.br>

History: 
	Jan 2004 Initial version

Return codes:
0	Finished test. See printed data to see if it was successful.

******************************************************************************/

#include <iostream>
#include "libglass.h"
#include <pthread.h>

pthread_t athreads;

using namespace libglass;

#define BARRIER_COUNT	9

using std::cerr;
using std::endl;

Glass *g;

void* thread_barrier1(void *arg)
	{
	Barrier *a= new Barrier(1,1);
	Barrier *b= new Barrier(2,1);
	Barrier *c= new Barrier(3,1);
	Barrier *d= new Barrier(4,1);
	Barrier *e= new Barrier(5,1);
	Barrier *f= new Barrier(6,1);
	Barrier *g= new Barrier(7,1);	
	sleep(6);

	int cont=0;
	while (cont <BARRIER_COUNT)		
		{
		cerr <<"Thread 1 -> Synch number="<<cont<< endl;
		a->sync();
		cerr <<"Barrier a syncronized " <<endl;
		b->sync();
		cerr <<"Barrier b syncronized " <<endl;
		c->sync();
		cerr <<"Barrier c syncronized " <<endl;		
		d->sync();
		cerr <<"Barrier d syncronized " <<endl;
		e->sync();
		cerr <<"Barrier e syncronized " <<endl;
		sleep(2);
		f->sync();
		cerr <<"Barrier f syncronized " <<endl;		
		g->sync();
		cerr <<"Barrier g syncronized " <<endl;
		cont++;
		}
	delete a;
	delete b;
	delete c;
	delete d;
	delete e;
	delete f;
	delete g;
	return NULL;
	}




int main(int argc, char **argv) {
	Glass		*g;
	int t;
	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}
        
	t = pthread_create(&athreads,NULL,thread_barrier1,NULL);
        pthread_join(athreads,NULL);

	delete g;

	return 0;
}
