/*  Glass - a distributed computing library
    Copyright (C) 2003-2009 Bruno Barberi Gnecco
    Copyright (C) 2009-2010 Corollarium Technologies


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/******************************************************************************
LibGlass thread safety tester.

Program description: 
	Tests thread safety in barriers and shared variables.

Dependencies:
	Glass, Barriers.

Authors:
	Marcelo de Paiva Guimaraes <marcelopaivag@yahoo.com.br>

History: 
	Jan 2004 Initial version

Return codes:
0	Finished test. See printed data to see if it was successful.

******************************************************************************/

#include <iostream>
#include "libglass.h"
#include <pthread.h>

pthread_t th_1,th_2;

#define BARRIER_COUNT	9

using namespace libglass;

using std::cerr;
using std::endl;

Glass *g;

Shared<int, rawPack, rawUnpack> *deg;

void* thread_barrier1(void *arg)
	{
	Barrier *b= new Barrier(1,false);
    Barrier *c= new Barrier(2,false);
	sleep(6);
	cerr << "Syncing Barrier..." << endl;
	if (b->sync() == false) {
		cerr << "Barrier failed 1..." << endl;
	//	return ;
		}	
	int cont=0;
	while (cont <BARRIER_COUNT)		
		{
		b->sync();
		cont++;
                *deg=**deg+1;
                deg->sendUpdate();
                deg->getUpdate();
                cerr <<"Thread 1 -> First  Barrier-> Shared variable value = "<< deg->getData()<<" Synch number="<<cont<< endl;		
                c->sync();
                cerr <<"Thread 1 -> Second Barrier-> Shared variable value = "<< deg->getData()<< endl;
//		sleep(2);
		}
	delete b;
	delete c;
	return NULL;
	}

void* thread_barrier2(void *arg)
	{
	Barrier *b= new Barrier(3,false);
    Barrier *c= new Barrier(4,false);
	sleep(6);
	cerr << "Syncing Barrier..." << endl;
	if (b->sync() == false) {
		cerr << "Barrier failed 2..." << endl;
//		return;
		}	
	int cont=0;
	while (cont <BARRIER_COUNT)		
		{
		b->sync();
                cerr <<"Thread 2 -> First  Barrier-> Shared variable value = "<< deg->getData()<<" Synch number="<<cont<< endl;
                c->sync();
                *deg=**deg+1;
                deg->sendUpdate();
                deg->getUpdate();
                cerr <<"Thread 2 -> Second Barrier-> Shared variable value = "<< deg->getData()<< endl;
		cont++;
//		sleep(1);		
	}
	delete b;
	delete c;
	return NULL;
	}



int main(int argc, char **argv) {
	Glass		*g;
	int t;
	
	try {
		if (argc == 1) {
			g = new GlassServer(new TCP());
		}
		else {
			g = new GlassClient(new TCP(), argv[1]);
		}
	}
	catch (Exception e) {
		cerr << e.getMessage() << endl;
		exit(1);
	}
        /* create our shared data, and set its value */
        deg = new Shared<int, rawPack, rawUnpack>("deg");
                                                                                                                        
        if (g->getNodeId() == 3) {
                *deg = 0;
                deg->sendUpdate();
        }


        t = pthread_create(&th_1,NULL,thread_barrier1,NULL);
	t = pthread_create(&th_2,NULL,thread_barrier2,NULL);
	
	pthread_join(th_1, NULL);
	pthread_join(th_2, NULL);

  //      while (true){sleep(20);}

	delete g;

	return 0;
}
